# -*- coding: utf-8 -*-
# **********************************************************************
#
# Copyright (c) 2003-2017 ZeroC, Inc. All rights reserved.
#
# This copy of Ice is licensed to you under the terms described in the
# ICE_LICENSE file included in this distribution.
#
# **********************************************************************
#
# Ice version 3.6.4
#
# <auto-generated>
#
# Generated from file `IPixels.ice'
#
# Warning: do not edit this file.
#
# </auto-generated>
#

from sys import version_info as _version_info_
import Ice, IcePy
import omero_Collections_ice
import omero_ServicesF_ice

# Included module omero
_M_omero = Ice.openModule('omero')

# Included module omero.model
_M_omero.model = Ice.openModule('omero.model')

# Included module Ice
_M_Ice = Ice.openModule('Ice')

# Included module omero.sys
_M_omero.sys = Ice.openModule('omero.sys')

# Included module omero.api
_M_omero.api = Ice.openModule('omero.api')

# Included module Glacier2
_M_Glacier2 = Ice.openModule('Glacier2')

# Included module omero.grid
_M_omero.grid = Ice.openModule('omero.grid')

# Start of module omero
__name__ = 'omero'

# Start of module omero.api
__name__ = 'omero.api'

if 'IPixels' not in _M_omero.api.__dict__:
    _M_omero.api.IPixels = Ice.createTempClass()
    class IPixels(_M_omero.api.ServiceInterface):
        """
        Metadata gateway for the omero.api.RenderingEngine and
        clients. This service provides all DB access that the rendering
        engine needs as well as Pixels services to a client. It also allows
        the rendering  engine to also be run external to the server (e.g.
        client-side).
        """
        def __init__(self):
            if Ice.getType(self) == _M_omero.api.IPixels:
                raise RuntimeError('omero.api.IPixels is an abstract class')

        def ice_ids(self, current=None):
            return ('::Ice::Object', '::omero::api::IPixels', '::omero::api::ServiceInterface')

        def ice_id(self, current=None):
            return '::omero::api::IPixels'

        def ice_staticId():
            return '::omero::api::IPixels'
        ice_staticId = staticmethod(ice_staticId)

        def retrievePixDescription_async(self, _cb, pixId, current=None):
            """
            Retrieves the pixels metadata. The following objects are
            pre-linked:
            pixels.pixelsType
            pixels.pixelsDimensions
            pixels.channels
            pixels.channnels.statsInfo
            pixels.channnels.colorComponent
            pixels.channnels.logicalChannel
            pixels.channnels.logicalChannel.photometricInterpretation
            Arguments:
            _cb -- The asynchronous callback object.
            pixId -- Pixels id.
            current -- The Current object for the invocation.
            """
            pass

        def retrieveRndSettings_async(self, _cb, pixId, current=None):
            """
            Retrieves the rendering settings for a given pixels set and
            the currently logged in user. If the current user has no
            omero.model.RenderingDef, and the user is an
            administrator, then a omero.model.RenderingDef may
            be returned for the owner of the
            omero.model.Pixels. This matches the behavior of the
            Rendering service.
            The following objects will be pre-linked:
            renderingDef.quantization
            renderingDef.model
            renderingDef.waveRendering
            renderingDef.waveRendering.color
            renderingDef.waveRendering.family
            renderingDef.spatialDomainEnhancement
            Arguments:
            _cb -- The asynchronous callback object.
            pixId -- Pixels id.
            current -- The Current object for the invocation.
            """
            pass

        def retrieveRndSettingsFor_async(self, _cb, pixId, userId, current=None):
            """
            Retrieves the rendering settings for a given pixels set and
            the passed user. The following objects are pre-linked:
            renderingDef.quantization
            renderingDef.model
            renderingDef.waveRendering
            renderingDef.waveRendering.color
            renderingDef.waveRendering.family
            renderingDef.spatialDomainEnhancement
            Arguments:
            _cb -- The asynchronous callback object.
            pixId -- Pixels id.
            userId -- The id of the user.
            current -- The Current object for the invocation.
            """
            pass

        def retrieveAllRndSettings_async(self, _cb, pixId, userId, current=None):
            """
            Retrieves all the rendering settings for a given pixels set
            and the passed user. The following objects are pre-linked:
            renderingDef.quantization
            renderingDef.model
            renderingDef.waveRendering
            renderingDef.waveRendering.color
            renderingDef.waveRendering.family
            renderingDef.spatialDomainEnhancement
            Arguments:
            _cb -- The asynchronous callback object.
            pixId -- Pixels id.
            userId -- The id of the user.
            current -- The Current object for the invocation.
            """
            pass

        def loadRndSettings_async(self, _cb, renderingSettingsId, current=None):
            """
            Loads a specific set of rendering settings. The
            following objects are pre-linked:
            renderingDef.quantization
            renderingDef.model
            renderingDef.waveRendering
            renderingDef.waveRendering.color
            renderingDef.waveRendering.family
            renderingDef.spatialDomainEnhancement
            Arguments:
            _cb -- The asynchronous callback object.
            renderingSettingsId -- Rendering definition id.
            current -- The Current object for the invocation.
            Throws:
            ValidationException -- If no RenderingDef matches the ID renderingDefId.
            """
            pass

        def saveRndSettings_async(self, _cb, rndSettings, current=None):
            """
            Saves the specified rendering settings.
            Arguments:
            _cb -- The asynchronous callback object.
            rndSettings -- Rendering settings.
            current -- The Current object for the invocation.
            """
            pass

        def getBitDepth_async(self, _cb, type, current=None):
            """
            Bit depth for a given pixel type.
            Arguments:
            _cb -- The asynchronous callback object.
            type -- Pixels type.
            current -- The Current object for the invocation.
            """
            pass

        def getEnumeration_async(self, _cb, enumClass, value, current=None):
            """
            Retrieves a particular enumeration for a given enumeration
            class.
            Arguments:
            _cb -- The asynchronous callback object.
            enumClass -- Enumeration class.
            value -- Enumeration string value.
            current -- The Current object for the invocation.
            """
            pass

        def getAllEnumerations_async(self, _cb, enumClass, current=None):
            """
            Retrieves the exhaustive list of enumerations for a given
            enumeration class.
            Arguments:
            _cb -- The asynchronous callback object.
            enumClass -- Enumeration class.
            current -- The Current object for the invocation.
            """
            pass

        def copyAndResizePixels_async(self, _cb, pixelsId, sizeX, sizeY, sizeZ, sizeT, channelList, methodology, copyStats, current=None):
            """
            Copies the metadata, and only the metadata linked to
            a Pixels object into a new Pixels object of equal or
            differing size across one or many of its three physical
            dimensions or temporal dimension.
            It is beyond the scope of this method to handle updates or
            changes to the raw pixel data available through
            omero.api.RawPixelsStore or to add
            and link omero.model.PlaneInfo and/or other Pixels
            set specific metadata.
            It is also assumed that the caller wishes the pixels
            dimensions and omero.model.PixelsType to remain the
            same; changing these is outside the scope of this method.
            NOTE: As omero.model.Channel objects are
            only able to apply to a single set of Pixels any
            annotations or linkage to these objects will be lost.
            Arguments:
            _cb -- The asynchronous callback object.
            pixelsId -- The source Pixels set id.
            sizeX -- The new size across the X-axis. null if the copy should maintain the same size.
            sizeY -- The new size across the Y-axis. null if the copy should maintain the same size.
            sizeZ -- The new size across the Z-axis. null if the copy should maintain the same size.
            sizeT -- The new number of timepoints. null if the copy should maintain the same number.
            channelList -- The channels that should be copied into the new Pixels set.
            methodology -- An optional string signifying the methodology that will be used to produce this new Pixels set.
            copyStats -- Whether or not to copy the omero.model.StatsInfo for each channel.
            current -- The Current object for the invocation.
            Throws:
            ValidationException -- If the X, Y, Z, T or channelList dimensions are out of bounds or the Pixels object corresponding to pixelsId is unlocatable.
            """
            pass

        def copyAndResizeImage_async(self, _cb, imageId, sizeX, sizeY, sizeZ, sizeT, channelList, methodology, copyStats, current=None):
            """
            Copies the metadata, and only the metadata linked to
            a Image object into a new Image object of equal or
            differing size across one or many of its three physical
            dimensions or temporal dimension.
            It is beyond the scope of this method to handle updates or
            changes to  the raw pixel data available through
            omero.api.RawPixelsStore or to add
            and link omero.model.PlaneInfo and/or other Pixels
            set specific metadata.
            It is also assumed that the caller wishes the pixels
            dimensions and omero.model.PixelsType to remain the
            same; changing these is outside the scope of this method.
            NOTE: As omero.model.Channel objects are
            only able to apply to a single set of Pixels any
            annotations or linkage to these objects will be lost.
            Arguments:
            _cb -- The asynchronous callback object.
            imageId -- The source Image id.
            sizeX -- The new size across the X-axis. null if the copy should maintain the same size.
            sizeY -- The new size across the Y-axis. null if the copy should maintain the same size.
            sizeZ -- The new size across the Z-axis. null if the copy should maintain the same size.
            sizeT -- The new number of timepoints. null if the copy should maintain the same number.
            channelList -- The channels that should be copied into the new Pixels set.
            methodology -- The name of the new Image.
            copyStats -- Whether or not to copy the omero.model.StatsInfo for each channel.
            current -- The Current object for the invocation.
            Throws:
            ValidationException -- If the X, Y, Z, T or channelList dimensions are out of bounds or the Pixels object corresponding to pixelsId is unlocatable.
            """
            pass

        def createImage_async(self, _cb, sizeX, sizeY, sizeZ, sizeT, channelList, pixelsType, name, description, current=None):
            """
            Creates the metadata, and only the metadata linked
            to an Image object. It is beyond the scope of this method
            to handle updates or changes to the raw pixel data
            available through omero.api.RawPixelsStore or to
            add and link omero.model.PlaneInfo or
            omero.model.StatsInfo objects and/or other Pixels
            set specific metadata. It is also up to the caller to
            update the pixels dimensions.
            Arguments:
            _cb -- The asynchronous callback object.
            sizeX -- The new size across the X-axis.
            sizeY -- The new size across the Y-axis.
            sizeZ -- The new size across the Z-axis.
            sizeT -- The new number of timepoints.
            channelList -- 
            pixelsType -- The pixelsType
            name -- The name of the new Image.
            description -- The description of the new Image.
            current -- The Current object for the invocation.
            Throws:
            ValidationException -- If the channel list is null or of size == 0.
            """
            pass

        def setChannelGlobalMinMax_async(self, _cb, pixelsId, channelIndex, min, max, current=None):
            """
            Sets the channel global (all 2D optical sections
            corresponding to a particular channel) minimum and maximum
            for a Pixels set.
            Arguments:
            _cb -- The asynchronous callback object.
            pixelsId -- The source Pixels set id.
            channelIndex -- The channel index within the Pixels set.
            min -- The channel global minimum.
            max -- The channel global maximum.
            current -- The Current object for the invocation.
            """
            pass

        def __str__(self):
            return IcePy.stringify(self, _M_omero.api._t_IPixels)

        __repr__ = __str__

    _M_omero.api.IPixelsPrx = Ice.createTempClass()
    class IPixelsPrx(_M_omero.api.ServiceInterfacePrx):

        """
        Retrieves the pixels metadata. The following objects are
        pre-linked:
        pixels.pixelsType
        pixels.pixelsDimensions
        pixels.channels
        pixels.channnels.statsInfo
        pixels.channnels.colorComponent
        pixels.channnels.logicalChannel
        pixels.channnels.logicalChannel.photometricInterpretation
        Arguments:
        pixId -- Pixels id.
        _ctx -- The request context for the invocation.
        Returns: Pixels object which matches id.
        """
        def retrievePixDescription(self, pixId, _ctx=None):
            return _M_omero.api.IPixels._op_retrievePixDescription.invoke(self, ((pixId, ), _ctx))

        """
        Retrieves the pixels metadata. The following objects are
        pre-linked:
        pixels.pixelsType
        pixels.pixelsDimensions
        pixels.channels
        pixels.channnels.statsInfo
        pixels.channnels.colorComponent
        pixels.channnels.logicalChannel
        pixels.channnels.logicalChannel.photometricInterpretation
        Arguments:
        pixId -- Pixels id.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_retrievePixDescription(self, pixId, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.IPixels._op_retrievePixDescription.begin(self, ((pixId, ), _response, _ex, _sent, _ctx))

        """
        Retrieves the pixels metadata. The following objects are
        pre-linked:
        pixels.pixelsType
        pixels.pixelsDimensions
        pixels.channels
        pixels.channnels.statsInfo
        pixels.channnels.colorComponent
        pixels.channnels.logicalChannel
        pixels.channnels.logicalChannel.photometricInterpretation
        Arguments:
        pixId -- Pixels id.
        Returns: Pixels object which matches id.
        """
        def end_retrievePixDescription(self, _r):
            return _M_omero.api.IPixels._op_retrievePixDescription.end(self, _r)

        """
        Retrieves the rendering settings for a given pixels set and
        the currently logged in user. If the current user has no
        omero.model.RenderingDef, and the user is an
        administrator, then a omero.model.RenderingDef may
        be returned for the owner of the
        omero.model.Pixels. This matches the behavior of the
        Rendering service.
        The following objects will be pre-linked:
        renderingDef.quantization
        renderingDef.model
        renderingDef.waveRendering
        renderingDef.waveRendering.color
        renderingDef.waveRendering.family
        renderingDef.spatialDomainEnhancement
        Arguments:
        pixId -- Pixels id.
        _ctx -- The request context for the invocation.
        Returns: Rendering definition.
        """
        def retrieveRndSettings(self, pixId, _ctx=None):
            return _M_omero.api.IPixels._op_retrieveRndSettings.invoke(self, ((pixId, ), _ctx))

        """
        Retrieves the rendering settings for a given pixels set and
        the currently logged in user. If the current user has no
        omero.model.RenderingDef, and the user is an
        administrator, then a omero.model.RenderingDef may
        be returned for the owner of the
        omero.model.Pixels. This matches the behavior of the
        Rendering service.
        The following objects will be pre-linked:
        renderingDef.quantization
        renderingDef.model
        renderingDef.waveRendering
        renderingDef.waveRendering.color
        renderingDef.waveRendering.family
        renderingDef.spatialDomainEnhancement
        Arguments:
        pixId -- Pixels id.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_retrieveRndSettings(self, pixId, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.IPixels._op_retrieveRndSettings.begin(self, ((pixId, ), _response, _ex, _sent, _ctx))

        """
        Retrieves the rendering settings for a given pixels set and
        the currently logged in user. If the current user has no
        omero.model.RenderingDef, and the user is an
        administrator, then a omero.model.RenderingDef may
        be returned for the owner of the
        omero.model.Pixels. This matches the behavior of the
        Rendering service.
        The following objects will be pre-linked:
        renderingDef.quantization
        renderingDef.model
        renderingDef.waveRendering
        renderingDef.waveRendering.color
        renderingDef.waveRendering.family
        renderingDef.spatialDomainEnhancement
        Arguments:
        pixId -- Pixels id.
        Returns: Rendering definition.
        """
        def end_retrieveRndSettings(self, _r):
            return _M_omero.api.IPixels._op_retrieveRndSettings.end(self, _r)

        """
        Retrieves the rendering settings for a given pixels set and
        the passed user. The following objects are pre-linked:
        renderingDef.quantization
        renderingDef.model
        renderingDef.waveRendering
        renderingDef.waveRendering.color
        renderingDef.waveRendering.family
        renderingDef.spatialDomainEnhancement
        Arguments:
        pixId -- Pixels id.
        userId -- The id of the user.
        _ctx -- The request context for the invocation.
        Returns: Rendering definition.
        """
        def retrieveRndSettingsFor(self, pixId, userId, _ctx=None):
            return _M_omero.api.IPixels._op_retrieveRndSettingsFor.invoke(self, ((pixId, userId), _ctx))

        """
        Retrieves the rendering settings for a given pixels set and
        the passed user. The following objects are pre-linked:
        renderingDef.quantization
        renderingDef.model
        renderingDef.waveRendering
        renderingDef.waveRendering.color
        renderingDef.waveRendering.family
        renderingDef.spatialDomainEnhancement
        Arguments:
        pixId -- Pixels id.
        userId -- The id of the user.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_retrieveRndSettingsFor(self, pixId, userId, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.IPixels._op_retrieveRndSettingsFor.begin(self, ((pixId, userId), _response, _ex, _sent, _ctx))

        """
        Retrieves the rendering settings for a given pixels set and
        the passed user. The following objects are pre-linked:
        renderingDef.quantization
        renderingDef.model
        renderingDef.waveRendering
        renderingDef.waveRendering.color
        renderingDef.waveRendering.family
        renderingDef.spatialDomainEnhancement
        Arguments:
        pixId -- Pixels id.
        userId -- The id of the user.
        Returns: Rendering definition.
        """
        def end_retrieveRndSettingsFor(self, _r):
            return _M_omero.api.IPixels._op_retrieveRndSettingsFor.end(self, _r)

        """
        Retrieves all the rendering settings for a given pixels set
        and the passed user. The following objects are pre-linked:
        renderingDef.quantization
        renderingDef.model
        renderingDef.waveRendering
        renderingDef.waveRendering.color
        renderingDef.waveRendering.family
        renderingDef.spatialDomainEnhancement
        Arguments:
        pixId -- Pixels id.
        userId -- The id of the user.
        _ctx -- The request context for the invocation.
        Returns: Rendering definition.
        """
        def retrieveAllRndSettings(self, pixId, userId, _ctx=None):
            return _M_omero.api.IPixels._op_retrieveAllRndSettings.invoke(self, ((pixId, userId), _ctx))

        """
        Retrieves all the rendering settings for a given pixels set
        and the passed user. The following objects are pre-linked:
        renderingDef.quantization
        renderingDef.model
        renderingDef.waveRendering
        renderingDef.waveRendering.color
        renderingDef.waveRendering.family
        renderingDef.spatialDomainEnhancement
        Arguments:
        pixId -- Pixels id.
        userId -- The id of the user.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_retrieveAllRndSettings(self, pixId, userId, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.IPixels._op_retrieveAllRndSettings.begin(self, ((pixId, userId), _response, _ex, _sent, _ctx))

        """
        Retrieves all the rendering settings for a given pixels set
        and the passed user. The following objects are pre-linked:
        renderingDef.quantization
        renderingDef.model
        renderingDef.waveRendering
        renderingDef.waveRendering.color
        renderingDef.waveRendering.family
        renderingDef.spatialDomainEnhancement
        Arguments:
        pixId -- Pixels id.
        userId -- The id of the user.
        Returns: Rendering definition.
        """
        def end_retrieveAllRndSettings(self, _r):
            return _M_omero.api.IPixels._op_retrieveAllRndSettings.end(self, _r)

        """
        Loads a specific set of rendering settings. The
        following objects are pre-linked:
        renderingDef.quantization
        renderingDef.model
        renderingDef.waveRendering
        renderingDef.waveRendering.color
        renderingDef.waveRendering.family
        renderingDef.spatialDomainEnhancement
        Arguments:
        renderingSettingsId -- Rendering definition id.
        _ctx -- The request context for the invocation.
        Returns: Rendering definition.
        Throws:
        ValidationException -- If no RenderingDef matches the ID renderingDefId.
        """
        def loadRndSettings(self, renderingSettingsId, _ctx=None):
            return _M_omero.api.IPixels._op_loadRndSettings.invoke(self, ((renderingSettingsId, ), _ctx))

        """
        Loads a specific set of rendering settings. The
        following objects are pre-linked:
        renderingDef.quantization
        renderingDef.model
        renderingDef.waveRendering
        renderingDef.waveRendering.color
        renderingDef.waveRendering.family
        renderingDef.spatialDomainEnhancement
        Arguments:
        renderingSettingsId -- Rendering definition id.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_loadRndSettings(self, renderingSettingsId, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.IPixels._op_loadRndSettings.begin(self, ((renderingSettingsId, ), _response, _ex, _sent, _ctx))

        """
        Loads a specific set of rendering settings. The
        following objects are pre-linked:
        renderingDef.quantization
        renderingDef.model
        renderingDef.waveRendering
        renderingDef.waveRendering.color
        renderingDef.waveRendering.family
        renderingDef.spatialDomainEnhancement
        Arguments:
        renderingSettingsId -- Rendering definition id.
        Returns: Rendering definition.
        Throws:
        ValidationException -- If no RenderingDef matches the ID renderingDefId.
        """
        def end_loadRndSettings(self, _r):
            return _M_omero.api.IPixels._op_loadRndSettings.end(self, _r)

        """
        Saves the specified rendering settings.
        Arguments:
        rndSettings -- Rendering settings.
        _ctx -- The request context for the invocation.
        """
        def saveRndSettings(self, rndSettings, _ctx=None):
            return _M_omero.api.IPixels._op_saveRndSettings.invoke(self, ((rndSettings, ), _ctx))

        """
        Saves the specified rendering settings.
        Arguments:
        rndSettings -- Rendering settings.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_saveRndSettings(self, rndSettings, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.IPixels._op_saveRndSettings.begin(self, ((rndSettings, ), _response, _ex, _sent, _ctx))

        """
        Saves the specified rendering settings.
        Arguments:
        rndSettings -- Rendering settings.
        """
        def end_saveRndSettings(self, _r):
            return _M_omero.api.IPixels._op_saveRndSettings.end(self, _r)

        """
        Bit depth for a given pixel type.
        Arguments:
        type -- Pixels type.
        _ctx -- The request context for the invocation.
        Returns: Bit depth in bits.
        """
        def getBitDepth(self, type, _ctx=None):
            return _M_omero.api.IPixels._op_getBitDepth.invoke(self, ((type, ), _ctx))

        """
        Bit depth for a given pixel type.
        Arguments:
        type -- Pixels type.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getBitDepth(self, type, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.IPixels._op_getBitDepth.begin(self, ((type, ), _response, _ex, _sent, _ctx))

        """
        Bit depth for a given pixel type.
        Arguments:
        type -- Pixels type.
        Returns: Bit depth in bits.
        """
        def end_getBitDepth(self, _r):
            return _M_omero.api.IPixels._op_getBitDepth.end(self, _r)

        """
        Retrieves a particular enumeration for a given enumeration
        class.
        Arguments:
        enumClass -- Enumeration class.
        value -- Enumeration string value.
        _ctx -- The request context for the invocation.
        Returns: Enumeration object.
        """
        def getEnumeration(self, enumClass, value, _ctx=None):
            return _M_omero.api.IPixels._op_getEnumeration.invoke(self, ((enumClass, value), _ctx))

        """
        Retrieves a particular enumeration for a given enumeration
        class.
        Arguments:
        enumClass -- Enumeration class.
        value -- Enumeration string value.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getEnumeration(self, enumClass, value, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.IPixels._op_getEnumeration.begin(self, ((enumClass, value), _response, _ex, _sent, _ctx))

        """
        Retrieves a particular enumeration for a given enumeration
        class.
        Arguments:
        enumClass -- Enumeration class.
        value -- Enumeration string value.
        Returns: Enumeration object.
        """
        def end_getEnumeration(self, _r):
            return _M_omero.api.IPixels._op_getEnumeration.end(self, _r)

        """
        Retrieves the exhaustive list of enumerations for a given
        enumeration class.
        Arguments:
        enumClass -- Enumeration class.
        _ctx -- The request context for the invocation.
        Returns: List of all enumeration objects for the enumClass.
        """
        def getAllEnumerations(self, enumClass, _ctx=None):
            return _M_omero.api.IPixels._op_getAllEnumerations.invoke(self, ((enumClass, ), _ctx))

        """
        Retrieves the exhaustive list of enumerations for a given
        enumeration class.
        Arguments:
        enumClass -- Enumeration class.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getAllEnumerations(self, enumClass, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.IPixels._op_getAllEnumerations.begin(self, ((enumClass, ), _response, _ex, _sent, _ctx))

        """
        Retrieves the exhaustive list of enumerations for a given
        enumeration class.
        Arguments:
        enumClass -- Enumeration class.
        Returns: List of all enumeration objects for the enumClass.
        """
        def end_getAllEnumerations(self, _r):
            return _M_omero.api.IPixels._op_getAllEnumerations.end(self, _r)

        """
        Copies the metadata, and only the metadata linked to
        a Pixels object into a new Pixels object of equal or
        differing size across one or many of its three physical
        dimensions or temporal dimension.
        It is beyond the scope of this method to handle updates or
        changes to the raw pixel data available through
        omero.api.RawPixelsStore or to add
        and link omero.model.PlaneInfo and/or other Pixels
        set specific metadata.
        It is also assumed that the caller wishes the pixels
        dimensions and omero.model.PixelsType to remain the
        same; changing these is outside the scope of this method.
        NOTE: As omero.model.Channel objects are
        only able to apply to a single set of Pixels any
        annotations or linkage to these objects will be lost.
        Arguments:
        pixelsId -- The source Pixels set id.
        sizeX -- The new size across the X-axis. null if the copy should maintain the same size.
        sizeY -- The new size across the Y-axis. null if the copy should maintain the same size.
        sizeZ -- The new size across the Z-axis. null if the copy should maintain the same size.
        sizeT -- The new number of timepoints. null if the copy should maintain the same number.
        channelList -- The channels that should be copied into the new Pixels set.
        methodology -- An optional string signifying the methodology that will be used to produce this new Pixels set.
        copyStats -- Whether or not to copy the omero.model.StatsInfo for each channel.
        _ctx -- The request context for the invocation.
        Returns: Id of the new Pixels object on success or null on failure.
        Throws:
        ValidationException -- If the X, Y, Z, T or channelList dimensions are out of bounds or the Pixels object corresponding to pixelsId is unlocatable.
        """
        def copyAndResizePixels(self, pixelsId, sizeX, sizeY, sizeZ, sizeT, channelList, methodology, copyStats, _ctx=None):
            return _M_omero.api.IPixels._op_copyAndResizePixels.invoke(self, ((pixelsId, sizeX, sizeY, sizeZ, sizeT, channelList, methodology, copyStats), _ctx))

        """
        Copies the metadata, and only the metadata linked to
        a Pixels object into a new Pixels object of equal or
        differing size across one or many of its three physical
        dimensions or temporal dimension.
        It is beyond the scope of this method to handle updates or
        changes to the raw pixel data available through
        omero.api.RawPixelsStore or to add
        and link omero.model.PlaneInfo and/or other Pixels
        set specific metadata.
        It is also assumed that the caller wishes the pixels
        dimensions and omero.model.PixelsType to remain the
        same; changing these is outside the scope of this method.
        NOTE: As omero.model.Channel objects are
        only able to apply to a single set of Pixels any
        annotations or linkage to these objects will be lost.
        Arguments:
        pixelsId -- The source Pixels set id.
        sizeX -- The new size across the X-axis. null if the copy should maintain the same size.
        sizeY -- The new size across the Y-axis. null if the copy should maintain the same size.
        sizeZ -- The new size across the Z-axis. null if the copy should maintain the same size.
        sizeT -- The new number of timepoints. null if the copy should maintain the same number.
        channelList -- The channels that should be copied into the new Pixels set.
        methodology -- An optional string signifying the methodology that will be used to produce this new Pixels set.
        copyStats -- Whether or not to copy the omero.model.StatsInfo for each channel.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_copyAndResizePixels(self, pixelsId, sizeX, sizeY, sizeZ, sizeT, channelList, methodology, copyStats, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.IPixels._op_copyAndResizePixels.begin(self, ((pixelsId, sizeX, sizeY, sizeZ, sizeT, channelList, methodology, copyStats), _response, _ex, _sent, _ctx))

        """
        Copies the metadata, and only the metadata linked to
        a Pixels object into a new Pixels object of equal or
        differing size across one or many of its three physical
        dimensions or temporal dimension.
        It is beyond the scope of this method to handle updates or
        changes to the raw pixel data available through
        omero.api.RawPixelsStore or to add
        and link omero.model.PlaneInfo and/or other Pixels
        set specific metadata.
        It is also assumed that the caller wishes the pixels
        dimensions and omero.model.PixelsType to remain the
        same; changing these is outside the scope of this method.
        NOTE: As omero.model.Channel objects are
        only able to apply to a single set of Pixels any
        annotations or linkage to these objects will be lost.
        Arguments:
        pixelsId -- The source Pixels set id.
        sizeX -- The new size across the X-axis. null if the copy should maintain the same size.
        sizeY -- The new size across the Y-axis. null if the copy should maintain the same size.
        sizeZ -- The new size across the Z-axis. null if the copy should maintain the same size.
        sizeT -- The new number of timepoints. null if the copy should maintain the same number.
        channelList -- The channels that should be copied into the new Pixels set.
        methodology -- An optional string signifying the methodology that will be used to produce this new Pixels set.
        copyStats -- Whether or not to copy the omero.model.StatsInfo for each channel.
        Returns: Id of the new Pixels object on success or null on failure.
        Throws:
        ValidationException -- If the X, Y, Z, T or channelList dimensions are out of bounds or the Pixels object corresponding to pixelsId is unlocatable.
        """
        def end_copyAndResizePixels(self, _r):
            return _M_omero.api.IPixels._op_copyAndResizePixels.end(self, _r)

        """
        Copies the metadata, and only the metadata linked to
        a Image object into a new Image object of equal or
        differing size across one or many of its three physical
        dimensions or temporal dimension.
        It is beyond the scope of this method to handle updates or
        changes to  the raw pixel data available through
        omero.api.RawPixelsStore or to add
        and link omero.model.PlaneInfo and/or other Pixels
        set specific metadata.
        It is also assumed that the caller wishes the pixels
        dimensions and omero.model.PixelsType to remain the
        same; changing these is outside the scope of this method.
        NOTE: As omero.model.Channel objects are
        only able to apply to a single set of Pixels any
        annotations or linkage to these objects will be lost.
        Arguments:
        imageId -- The source Image id.
        sizeX -- The new size across the X-axis. null if the copy should maintain the same size.
        sizeY -- The new size across the Y-axis. null if the copy should maintain the same size.
        sizeZ -- The new size across the Z-axis. null if the copy should maintain the same size.
        sizeT -- The new number of timepoints. null if the copy should maintain the same number.
        channelList -- The channels that should be copied into the new Pixels set.
        methodology -- The name of the new Image.
        copyStats -- Whether or not to copy the omero.model.StatsInfo for each channel.
        _ctx -- The request context for the invocation.
        Returns: Id of the new Pixels object on success or null on failure.
        Throws:
        ValidationException -- If the X, Y, Z, T or channelList dimensions are out of bounds or the Pixels object corresponding to pixelsId is unlocatable.
        """
        def copyAndResizeImage(self, imageId, sizeX, sizeY, sizeZ, sizeT, channelList, methodology, copyStats, _ctx=None):
            return _M_omero.api.IPixels._op_copyAndResizeImage.invoke(self, ((imageId, sizeX, sizeY, sizeZ, sizeT, channelList, methodology, copyStats), _ctx))

        """
        Copies the metadata, and only the metadata linked to
        a Image object into a new Image object of equal or
        differing size across one or many of its three physical
        dimensions or temporal dimension.
        It is beyond the scope of this method to handle updates or
        changes to  the raw pixel data available through
        omero.api.RawPixelsStore or to add
        and link omero.model.PlaneInfo and/or other Pixels
        set specific metadata.
        It is also assumed that the caller wishes the pixels
        dimensions and omero.model.PixelsType to remain the
        same; changing these is outside the scope of this method.
        NOTE: As omero.model.Channel objects are
        only able to apply to a single set of Pixels any
        annotations or linkage to these objects will be lost.
        Arguments:
        imageId -- The source Image id.
        sizeX -- The new size across the X-axis. null if the copy should maintain the same size.
        sizeY -- The new size across the Y-axis. null if the copy should maintain the same size.
        sizeZ -- The new size across the Z-axis. null if the copy should maintain the same size.
        sizeT -- The new number of timepoints. null if the copy should maintain the same number.
        channelList -- The channels that should be copied into the new Pixels set.
        methodology -- The name of the new Image.
        copyStats -- Whether or not to copy the omero.model.StatsInfo for each channel.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_copyAndResizeImage(self, imageId, sizeX, sizeY, sizeZ, sizeT, channelList, methodology, copyStats, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.IPixels._op_copyAndResizeImage.begin(self, ((imageId, sizeX, sizeY, sizeZ, sizeT, channelList, methodology, copyStats), _response, _ex, _sent, _ctx))

        """
        Copies the metadata, and only the metadata linked to
        a Image object into a new Image object of equal or
        differing size across one or many of its three physical
        dimensions or temporal dimension.
        It is beyond the scope of this method to handle updates or
        changes to  the raw pixel data available through
        omero.api.RawPixelsStore or to add
        and link omero.model.PlaneInfo and/or other Pixels
        set specific metadata.
        It is also assumed that the caller wishes the pixels
        dimensions and omero.model.PixelsType to remain the
        same; changing these is outside the scope of this method.
        NOTE: As omero.model.Channel objects are
        only able to apply to a single set of Pixels any
        annotations or linkage to these objects will be lost.
        Arguments:
        imageId -- The source Image id.
        sizeX -- The new size across the X-axis. null if the copy should maintain the same size.
        sizeY -- The new size across the Y-axis. null if the copy should maintain the same size.
        sizeZ -- The new size across the Z-axis. null if the copy should maintain the same size.
        sizeT -- The new number of timepoints. null if the copy should maintain the same number.
        channelList -- The channels that should be copied into the new Pixels set.
        methodology -- The name of the new Image.
        copyStats -- Whether or not to copy the omero.model.StatsInfo for each channel.
        Returns: Id of the new Pixels object on success or null on failure.
        Throws:
        ValidationException -- If the X, Y, Z, T or channelList dimensions are out of bounds or the Pixels object corresponding to pixelsId is unlocatable.
        """
        def end_copyAndResizeImage(self, _r):
            return _M_omero.api.IPixels._op_copyAndResizeImage.end(self, _r)

        """
        Creates the metadata, and only the metadata linked
        to an Image object. It is beyond the scope of this method
        to handle updates or changes to the raw pixel data
        available through omero.api.RawPixelsStore or to
        add and link omero.model.PlaneInfo or
        omero.model.StatsInfo objects and/or other Pixels
        set specific metadata. It is also up to the caller to
        update the pixels dimensions.
        Arguments:
        sizeX -- The new size across the X-axis.
        sizeY -- The new size across the Y-axis.
        sizeZ -- The new size across the Z-axis.
        sizeT -- The new number of timepoints.
        channelList -- 
        pixelsType -- The pixelsType
        name -- The name of the new Image.
        description -- The description of the new Image.
        _ctx -- The request context for the invocation.
        Returns: Id of the new Image object on success or null on failure.
        Throws:
        ValidationException -- If the channel list is null or of size == 0.
        """
        def createImage(self, sizeX, sizeY, sizeZ, sizeT, channelList, pixelsType, name, description, _ctx=None):
            return _M_omero.api.IPixels._op_createImage.invoke(self, ((sizeX, sizeY, sizeZ, sizeT, channelList, pixelsType, name, description), _ctx))

        """
        Creates the metadata, and only the metadata linked
        to an Image object. It is beyond the scope of this method
        to handle updates or changes to the raw pixel data
        available through omero.api.RawPixelsStore or to
        add and link omero.model.PlaneInfo or
        omero.model.StatsInfo objects and/or other Pixels
        set specific metadata. It is also up to the caller to
        update the pixels dimensions.
        Arguments:
        sizeX -- The new size across the X-axis.
        sizeY -- The new size across the Y-axis.
        sizeZ -- The new size across the Z-axis.
        sizeT -- The new number of timepoints.
        channelList -- 
        pixelsType -- The pixelsType
        name -- The name of the new Image.
        description -- The description of the new Image.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_createImage(self, sizeX, sizeY, sizeZ, sizeT, channelList, pixelsType, name, description, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.IPixels._op_createImage.begin(self, ((sizeX, sizeY, sizeZ, sizeT, channelList, pixelsType, name, description), _response, _ex, _sent, _ctx))

        """
        Creates the metadata, and only the metadata linked
        to an Image object. It is beyond the scope of this method
        to handle updates or changes to the raw pixel data
        available through omero.api.RawPixelsStore or to
        add and link omero.model.PlaneInfo or
        omero.model.StatsInfo objects and/or other Pixels
        set specific metadata. It is also up to the caller to
        update the pixels dimensions.
        Arguments:
        sizeX -- The new size across the X-axis.
        sizeY -- The new size across the Y-axis.
        sizeZ -- The new size across the Z-axis.
        sizeT -- The new number of timepoints.
        channelList -- 
        pixelsType -- The pixelsType
        name -- The name of the new Image.
        description -- The description of the new Image.
        Returns: Id of the new Image object on success or null on failure.
        Throws:
        ValidationException -- If the channel list is null or of size == 0.
        """
        def end_createImage(self, _r):
            return _M_omero.api.IPixels._op_createImage.end(self, _r)

        """
        Sets the channel global (all 2D optical sections
        corresponding to a particular channel) minimum and maximum
        for a Pixels set.
        Arguments:
        pixelsId -- The source Pixels set id.
        channelIndex -- The channel index within the Pixels set.
        min -- The channel global minimum.
        max -- The channel global maximum.
        _ctx -- The request context for the invocation.
        """
        def setChannelGlobalMinMax(self, pixelsId, channelIndex, min, max, _ctx=None):
            return _M_omero.api.IPixels._op_setChannelGlobalMinMax.invoke(self, ((pixelsId, channelIndex, min, max), _ctx))

        """
        Sets the channel global (all 2D optical sections
        corresponding to a particular channel) minimum and maximum
        for a Pixels set.
        Arguments:
        pixelsId -- The source Pixels set id.
        channelIndex -- The channel index within the Pixels set.
        min -- The channel global minimum.
        max -- The channel global maximum.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_setChannelGlobalMinMax(self, pixelsId, channelIndex, min, max, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.IPixels._op_setChannelGlobalMinMax.begin(self, ((pixelsId, channelIndex, min, max), _response, _ex, _sent, _ctx))

        """
        Sets the channel global (all 2D optical sections
        corresponding to a particular channel) minimum and maximum
        for a Pixels set.
        Arguments:
        pixelsId -- The source Pixels set id.
        channelIndex -- The channel index within the Pixels set.
        min -- The channel global minimum.
        max -- The channel global maximum.
        """
        def end_setChannelGlobalMinMax(self, _r):
            return _M_omero.api.IPixels._op_setChannelGlobalMinMax.end(self, _r)

        def checkedCast(proxy, facetOrCtx=None, _ctx=None):
            return _M_omero.api.IPixelsPrx.ice_checkedCast(proxy, '::omero::api::IPixels', facetOrCtx, _ctx)
        checkedCast = staticmethod(checkedCast)

        def uncheckedCast(proxy, facet=None):
            return _M_omero.api.IPixelsPrx.ice_uncheckedCast(proxy, facet)
        uncheckedCast = staticmethod(uncheckedCast)

        def ice_staticId():
            return '::omero::api::IPixels'
        ice_staticId = staticmethod(ice_staticId)

    _M_omero.api._t_IPixelsPrx = IcePy.defineProxy('::omero::api::IPixels', IPixelsPrx)

    _M_omero.api._t_IPixels = IcePy.defineClass('::omero::api::IPixels', IPixels, -1, (), True, False, None, (_M_omero.api._t_ServiceInterface,), ())
    IPixels._ice_type = _M_omero.api._t_IPixels

    IPixels._op_retrievePixDescription = IcePy.Operation('retrievePixDescription', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (((), IcePy._t_long, False, 0),), (), ((), _M_omero.model._t_Pixels, False, 0), (_M_omero._t_ServerError,))
    IPixels._op_retrieveRndSettings = IcePy.Operation('retrieveRndSettings', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (((), IcePy._t_long, False, 0),), (), ((), _M_omero.model._t_RenderingDef, False, 0), (_M_omero._t_ServerError,))
    IPixels._op_retrieveRndSettingsFor = IcePy.Operation('retrieveRndSettingsFor', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (((), IcePy._t_long, False, 0), ((), IcePy._t_long, False, 0)), (), ((), _M_omero.model._t_RenderingDef, False, 0), (_M_omero._t_ServerError,))
    IPixels._op_retrieveAllRndSettings = IcePy.Operation('retrieveAllRndSettings', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (((), IcePy._t_long, False, 0), ((), IcePy._t_long, False, 0)), (), ((), _M_omero.api._t_IObjectList, False, 0), (_M_omero._t_ServerError,))
    IPixels._op_loadRndSettings = IcePy.Operation('loadRndSettings', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (((), IcePy._t_long, False, 0),), (), ((), _M_omero.model._t_RenderingDef, False, 0), (_M_omero._t_ServerError,))
    IPixels._op_saveRndSettings = IcePy.Operation('saveRndSettings', Ice.OperationMode.Normal, Ice.OperationMode.Normal, True, None, (), (((), _M_omero.model._t_RenderingDef, False, 0),), (), None, (_M_omero._t_ServerError,))
    IPixels._op_getBitDepth = IcePy.Operation('getBitDepth', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (((), _M_omero.model._t_PixelsType, False, 0),), (), ((), IcePy._t_int, False, 0), (_M_omero._t_ServerError,))
    IPixels._op_getEnumeration = IcePy.Operation('getEnumeration', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (((), IcePy._t_string, False, 0), ((), IcePy._t_string, False, 0)), (), ((), _M_omero.model._t_IObject, False, 0), (_M_omero._t_ServerError,))
    IPixels._op_getEnumeration.deprecate("Use ITypes#getEnumeration(string, string) instead.")
    IPixels._op_getAllEnumerations = IcePy.Operation('getAllEnumerations', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (((), IcePy._t_string, False, 0),), (), ((), _M_omero.api._t_IObjectList, False, 0), (_M_omero._t_ServerError,))
    IPixels._op_getAllEnumerations.deprecate("Use ITypes#allEnumerations(string) instead.")
    IPixels._op_copyAndResizePixels = IcePy.Operation('copyAndResizePixels', Ice.OperationMode.Normal, Ice.OperationMode.Normal, True, None, (), (((), IcePy._t_long, False, 0), ((), _M_omero._t_RInt, False, 0), ((), _M_omero._t_RInt, False, 0), ((), _M_omero._t_RInt, False, 0), ((), _M_omero._t_RInt, False, 0), ((), _M_omero.sys._t_IntList, False, 0), ((), IcePy._t_string, False, 0), ((), IcePy._t_bool, False, 0)), (), ((), _M_omero._t_RLong, False, 0), (_M_omero._t_ServerError,))
    IPixels._op_copyAndResizeImage = IcePy.Operation('copyAndResizeImage', Ice.OperationMode.Normal, Ice.OperationMode.Normal, True, None, (), (((), IcePy._t_long, False, 0), ((), _M_omero._t_RInt, False, 0), ((), _M_omero._t_RInt, False, 0), ((), _M_omero._t_RInt, False, 0), ((), _M_omero._t_RInt, False, 0), ((), _M_omero.sys._t_IntList, False, 0), ((), IcePy._t_string, False, 0), ((), IcePy._t_bool, False, 0)), (), ((), _M_omero._t_RLong, False, 0), (_M_omero._t_ServerError,))
    IPixels._op_createImage = IcePy.Operation('createImage', Ice.OperationMode.Normal, Ice.OperationMode.Normal, True, None, (), (((), IcePy._t_int, False, 0), ((), IcePy._t_int, False, 0), ((), IcePy._t_int, False, 0), ((), IcePy._t_int, False, 0), ((), _M_omero.sys._t_IntList, False, 0), ((), _M_omero.model._t_PixelsType, False, 0), ((), IcePy._t_string, False, 0), ((), IcePy._t_string, False, 0)), (), ((), _M_omero._t_RLong, False, 0), (_M_omero._t_ServerError,))
    IPixels._op_setChannelGlobalMinMax = IcePy.Operation('setChannelGlobalMinMax', Ice.OperationMode.Normal, Ice.OperationMode.Normal, True, None, (), (((), IcePy._t_long, False, 0), ((), IcePy._t_int, False, 0), ((), IcePy._t_double, False, 0), ((), IcePy._t_double, False, 0)), (), None, (_M_omero._t_ServerError,))

    _M_omero.api.IPixels = IPixels
    del IPixels

    _M_omero.api.IPixelsPrx = IPixelsPrx
    del IPixelsPrx

# End of module omero.api

__name__ = 'omero'

# End of module omero



# -*- coding: utf-8 -*-
# **********************************************************************
#
# Copyright (c) 2003-2017 ZeroC, Inc. All rights reserved.
#
# This copy of Ice is licensed to you under the terms described in the
# ICE_LICENSE file included in this distribution.
#
# **********************************************************************
#
# Ice version 3.6.4
#
# <auto-generated>
#
# Generated from file `StageLabel.ice'
#
# Warning: do not edit this file.
#
# </auto-generated>
#

from sys import version_info as _version_info_
import Ice, IcePy
import omero_model_IObject_ice
import omero_RTypes_ice
import omero_model_RTypes_ice
import omero_System_ice
import omero_Collections_ice

# Included module omero
_M_omero = Ice.openModule('omero')

# Included module omero.model
_M_omero.model = Ice.openModule('omero.model')

# Included module Ice
_M_Ice = Ice.openModule('Ice')

# Included module omero.sys
_M_omero.sys = Ice.openModule('omero.sys')

# Included module omero.api
_M_omero.api = Ice.openModule('omero.api')

# Start of module omero
__name__ = 'omero'

# Start of module omero.model
__name__ = 'omero.model'

if 'Length' not in _M_omero.model.__dict__:
    _M_omero.model._t_Length = IcePy.declareClass('::omero::model::Length')
    _M_omero.model._t_LengthPrx = IcePy.declareProxy('::omero::model::Length')

if 'Details' not in _M_omero.model.__dict__:
    _M_omero.model._t_Details = IcePy.declareClass('::omero::model::Details')
    _M_omero.model._t_DetailsPrx = IcePy.declareProxy('::omero::model::Details')

if 'StageLabel' not in _M_omero.model.__dict__:
    _M_omero.model.StageLabel = Ice.createTempClass()
    class StageLabel(_M_omero.model.IObject):
        def __init__(self, _id=None, _details=None, _loaded=False, _version=None, _positionX=None, _positionY=None, _positionZ=None, _name=None):
            if Ice.getType(self) == _M_omero.model.StageLabel:
                raise RuntimeError('omero.model.StageLabel is an abstract class')
            _M_omero.model.IObject.__init__(self, _id, _details, _loaded)
            self._version = _version
            self._positionX = _positionX
            self._positionY = _positionY
            self._positionZ = _positionZ
            self._name = _name

        def ice_ids(self, current=None):
            return ('::Ice::Object', '::omero::model::IObject', '::omero::model::StageLabel')

        def ice_id(self, current=None):
            return '::omero::model::StageLabel'

        def ice_staticId():
            return '::omero::model::StageLabel'
        ice_staticId = staticmethod(ice_staticId)

        def getVersion(self, current=None):
            pass

        def setVersion(self, theVersion, current=None):
            pass

        def getPositionX(self, current=None):
            pass

        def setPositionX(self, thePositionX, current=None):
            pass

        def getPositionY(self, current=None):
            pass

        def setPositionY(self, thePositionY, current=None):
            pass

        def getPositionZ(self, current=None):
            pass

        def setPositionZ(self, thePositionZ, current=None):
            pass

        def getName(self, current=None):
            pass

        def setName(self, theName, current=None):
            pass

        def __str__(self):
            return IcePy.stringify(self, _M_omero.model._t_StageLabel)

        __repr__ = __str__

    _M_omero.model.StageLabelPrx = Ice.createTempClass()
    class StageLabelPrx(_M_omero.model.IObjectPrx):

        def getVersion(self, _ctx=None):
            return _M_omero.model.StageLabel._op_getVersion.invoke(self, ((), _ctx))

        def begin_getVersion(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.StageLabel._op_getVersion.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getVersion(self, _r):
            return _M_omero.model.StageLabel._op_getVersion.end(self, _r)

        def setVersion(self, theVersion, _ctx=None):
            return _M_omero.model.StageLabel._op_setVersion.invoke(self, ((theVersion, ), _ctx))

        def begin_setVersion(self, theVersion, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.StageLabel._op_setVersion.begin(self, ((theVersion, ), _response, _ex, _sent, _ctx))

        def end_setVersion(self, _r):
            return _M_omero.model.StageLabel._op_setVersion.end(self, _r)

        def getPositionX(self, _ctx=None):
            return _M_omero.model.StageLabel._op_getPositionX.invoke(self, ((), _ctx))

        def begin_getPositionX(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.StageLabel._op_getPositionX.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getPositionX(self, _r):
            return _M_omero.model.StageLabel._op_getPositionX.end(self, _r)

        def setPositionX(self, thePositionX, _ctx=None):
            return _M_omero.model.StageLabel._op_setPositionX.invoke(self, ((thePositionX, ), _ctx))

        def begin_setPositionX(self, thePositionX, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.StageLabel._op_setPositionX.begin(self, ((thePositionX, ), _response, _ex, _sent, _ctx))

        def end_setPositionX(self, _r):
            return _M_omero.model.StageLabel._op_setPositionX.end(self, _r)

        def getPositionY(self, _ctx=None):
            return _M_omero.model.StageLabel._op_getPositionY.invoke(self, ((), _ctx))

        def begin_getPositionY(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.StageLabel._op_getPositionY.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getPositionY(self, _r):
            return _M_omero.model.StageLabel._op_getPositionY.end(self, _r)

        def setPositionY(self, thePositionY, _ctx=None):
            return _M_omero.model.StageLabel._op_setPositionY.invoke(self, ((thePositionY, ), _ctx))

        def begin_setPositionY(self, thePositionY, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.StageLabel._op_setPositionY.begin(self, ((thePositionY, ), _response, _ex, _sent, _ctx))

        def end_setPositionY(self, _r):
            return _M_omero.model.StageLabel._op_setPositionY.end(self, _r)

        def getPositionZ(self, _ctx=None):
            return _M_omero.model.StageLabel._op_getPositionZ.invoke(self, ((), _ctx))

        def begin_getPositionZ(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.StageLabel._op_getPositionZ.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getPositionZ(self, _r):
            return _M_omero.model.StageLabel._op_getPositionZ.end(self, _r)

        def setPositionZ(self, thePositionZ, _ctx=None):
            return _M_omero.model.StageLabel._op_setPositionZ.invoke(self, ((thePositionZ, ), _ctx))

        def begin_setPositionZ(self, thePositionZ, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.StageLabel._op_setPositionZ.begin(self, ((thePositionZ, ), _response, _ex, _sent, _ctx))

        def end_setPositionZ(self, _r):
            return _M_omero.model.StageLabel._op_setPositionZ.end(self, _r)

        def getName(self, _ctx=None):
            return _M_omero.model.StageLabel._op_getName.invoke(self, ((), _ctx))

        def begin_getName(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.StageLabel._op_getName.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getName(self, _r):
            return _M_omero.model.StageLabel._op_getName.end(self, _r)

        def setName(self, theName, _ctx=None):
            return _M_omero.model.StageLabel._op_setName.invoke(self, ((theName, ), _ctx))

        def begin_setName(self, theName, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.StageLabel._op_setName.begin(self, ((theName, ), _response, _ex, _sent, _ctx))

        def end_setName(self, _r):
            return _M_omero.model.StageLabel._op_setName.end(self, _r)

        def checkedCast(proxy, facetOrCtx=None, _ctx=None):
            return _M_omero.model.StageLabelPrx.ice_checkedCast(proxy, '::omero::model::StageLabel', facetOrCtx, _ctx)
        checkedCast = staticmethod(checkedCast)

        def uncheckedCast(proxy, facet=None):
            return _M_omero.model.StageLabelPrx.ice_uncheckedCast(proxy, facet)
        uncheckedCast = staticmethod(uncheckedCast)

        def ice_staticId():
            return '::omero::model::StageLabel'
        ice_staticId = staticmethod(ice_staticId)

    _M_omero.model._t_StageLabelPrx = IcePy.defineProxy('::omero::model::StageLabel', StageLabelPrx)

    _M_omero.model._t_StageLabel = IcePy.declareClass('::omero::model::StageLabel')

    _M_omero.model._t_StageLabel = IcePy.defineClass('::omero::model::StageLabel', StageLabel, -1, (), True, False, _M_omero.model._t_IObject, (), (
        ('_version', (), _M_omero._t_RInt, False, 0),
        ('_positionX', (), _M_omero.model._t_Length, False, 0),
        ('_positionY', (), _M_omero.model._t_Length, False, 0),
        ('_positionZ', (), _M_omero.model._t_Length, False, 0),
        ('_name', (), _M_omero._t_RString, False, 0)
    ))
    StageLabel._ice_type = _M_omero.model._t_StageLabel

    StageLabel._op_getVersion = IcePy.Operation('getVersion', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero._t_RInt, False, 0), ())
    StageLabel._op_setVersion = IcePy.Operation('setVersion', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero._t_RInt, False, 0),), (), None, ())
    StageLabel._op_getPositionX = IcePy.Operation('getPositionX', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.model._t_Length, False, 0), ())
    StageLabel._op_setPositionX = IcePy.Operation('setPositionX', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Length, False, 0),), (), None, ())
    StageLabel._op_getPositionY = IcePy.Operation('getPositionY', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.model._t_Length, False, 0), ())
    StageLabel._op_setPositionY = IcePy.Operation('setPositionY', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Length, False, 0),), (), None, ())
    StageLabel._op_getPositionZ = IcePy.Operation('getPositionZ', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.model._t_Length, False, 0), ())
    StageLabel._op_setPositionZ = IcePy.Operation('setPositionZ', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Length, False, 0),), (), None, ())
    StageLabel._op_getName = IcePy.Operation('getName', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero._t_RString, False, 0), ())
    StageLabel._op_setName = IcePy.Operation('setName', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero._t_RString, False, 0),), (), None, ())

    _M_omero.model.StageLabel = StageLabel
    del StageLabel

    _M_omero.model.StageLabelPrx = StageLabelPrx
    del StageLabelPrx

# End of module omero.model

__name__ = 'omero'

# End of module omero



# -*- coding: utf-8 -*-
# **********************************************************************
#
# Copyright (c) 2003-2017 ZeroC, Inc. All rights reserved.
#
# This copy of Ice is licensed to you under the terms described in the
# ICE_LICENSE file included in this distribution.
#
# **********************************************************************
#
# Ice version 3.6.4
#
# <auto-generated>
#
# Generated from file `Polyline.ice'
#
# Warning: do not edit this file.
#
# </auto-generated>
#

from sys import version_info as _version_info_
import Ice, IcePy
import omero_model_IObject_ice
import omero_RTypes_ice
import omero_model_RTypes_ice
import omero_System_ice
import omero_Collections_ice
import omero_model_Shape_ice

# Included module omero
_M_omero = Ice.openModule('omero')

# Included module omero.model
_M_omero.model = Ice.openModule('omero.model')

# Included module Ice
_M_Ice = Ice.openModule('Ice')

# Included module omero.sys
_M_omero.sys = Ice.openModule('omero.sys')

# Included module omero.api
_M_omero.api = Ice.openModule('omero.api')

# Start of module omero
__name__ = 'omero'

# Start of module omero.model
__name__ = 'omero.model'

if 'Roi' not in _M_omero.model.__dict__:
    _M_omero.model._t_Roi = IcePy.declareClass('::omero::model::Roi')
    _M_omero.model._t_RoiPrx = IcePy.declareProxy('::omero::model::Roi')

if 'AffineTransform' not in _M_omero.model.__dict__:
    _M_omero.model._t_AffineTransform = IcePy.declareClass('::omero::model::AffineTransform')
    _M_omero.model._t_AffineTransformPrx = IcePy.declareProxy('::omero::model::AffineTransform')

if 'Length' not in _M_omero.model.__dict__:
    _M_omero.model._t_Length = IcePy.declareClass('::omero::model::Length')
    _M_omero.model._t_LengthPrx = IcePy.declareProxy('::omero::model::Length')

if 'ShapeAnnotationLink' not in _M_omero.model.__dict__:
    _M_omero.model._t_ShapeAnnotationLink = IcePy.declareClass('::omero::model::ShapeAnnotationLink')
    _M_omero.model._t_ShapeAnnotationLinkPrx = IcePy.declareProxy('::omero::model::ShapeAnnotationLink')

if 'Annotation' not in _M_omero.model.__dict__:
    _M_omero.model._t_Annotation = IcePy.declareClass('::omero::model::Annotation')
    _M_omero.model._t_AnnotationPrx = IcePy.declareProxy('::omero::model::Annotation')

if 'Details' not in _M_omero.model.__dict__:
    _M_omero.model._t_Details = IcePy.declareClass('::omero::model::Details')
    _M_omero.model._t_DetailsPrx = IcePy.declareProxy('::omero::model::Details')

if 'Polyline' not in _M_omero.model.__dict__:
    _M_omero.model.Polyline = Ice.createTempClass()
    class Polyline(_M_omero.model.Shape):
        def __init__(self, _id=None, _details=None, _loaded=False, _version=None, _theZ=None, _theT=None, _theC=None, _roi=None, _locked=None, _transform=None, _fillColor=None, _fillRule=None, _strokeColor=None, _strokeDashArray=None, _strokeWidth=None, _fontFamily=None, _fontSize=None, _fontStyle=None, _annotationLinksSeq=None, _annotationLinksLoaded=False, _annotationLinksCountPerOwner=None, _points=None, _markerStart=None, _markerEnd=None, _textValue=None):
            if Ice.getType(self) == _M_omero.model.Polyline:
                raise RuntimeError('omero.model.Polyline is an abstract class')
            _M_omero.model.Shape.__init__(self, _id, _details, _loaded, _version, _theZ, _theT, _theC, _roi, _locked, _transform, _fillColor, _fillRule, _strokeColor, _strokeDashArray, _strokeWidth, _fontFamily, _fontSize, _fontStyle, _annotationLinksSeq, _annotationLinksLoaded, _annotationLinksCountPerOwner)
            self._points = _points
            self._markerStart = _markerStart
            self._markerEnd = _markerEnd
            self._textValue = _textValue

        def ice_ids(self, current=None):
            return ('::Ice::Object', '::omero::model::IObject', '::omero::model::Polyline', '::omero::model::Shape')

        def ice_id(self, current=None):
            return '::omero::model::Polyline'

        def ice_staticId():
            return '::omero::model::Polyline'
        ice_staticId = staticmethod(ice_staticId)

        def getPoints(self, current=None):
            pass

        def setPoints(self, thePoints, current=None):
            pass

        def getMarkerStart(self, current=None):
            pass

        def setMarkerStart(self, theMarkerStart, current=None):
            pass

        def getMarkerEnd(self, current=None):
            pass

        def setMarkerEnd(self, theMarkerEnd, current=None):
            pass

        def getTextValue(self, current=None):
            pass

        def setTextValue(self, theTextValue, current=None):
            pass

        def __str__(self):
            return IcePy.stringify(self, _M_omero.model._t_Polyline)

        __repr__ = __str__

    _M_omero.model.PolylinePrx = Ice.createTempClass()
    class PolylinePrx(_M_omero.model.ShapePrx):

        def getPoints(self, _ctx=None):
            return _M_omero.model.Polyline._op_getPoints.invoke(self, ((), _ctx))

        def begin_getPoints(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Polyline._op_getPoints.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getPoints(self, _r):
            return _M_omero.model.Polyline._op_getPoints.end(self, _r)

        def setPoints(self, thePoints, _ctx=None):
            return _M_omero.model.Polyline._op_setPoints.invoke(self, ((thePoints, ), _ctx))

        def begin_setPoints(self, thePoints, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Polyline._op_setPoints.begin(self, ((thePoints, ), _response, _ex, _sent, _ctx))

        def end_setPoints(self, _r):
            return _M_omero.model.Polyline._op_setPoints.end(self, _r)

        def getMarkerStart(self, _ctx=None):
            return _M_omero.model.Polyline._op_getMarkerStart.invoke(self, ((), _ctx))

        def begin_getMarkerStart(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Polyline._op_getMarkerStart.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getMarkerStart(self, _r):
            return _M_omero.model.Polyline._op_getMarkerStart.end(self, _r)

        def setMarkerStart(self, theMarkerStart, _ctx=None):
            return _M_omero.model.Polyline._op_setMarkerStart.invoke(self, ((theMarkerStart, ), _ctx))

        def begin_setMarkerStart(self, theMarkerStart, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Polyline._op_setMarkerStart.begin(self, ((theMarkerStart, ), _response, _ex, _sent, _ctx))

        def end_setMarkerStart(self, _r):
            return _M_omero.model.Polyline._op_setMarkerStart.end(self, _r)

        def getMarkerEnd(self, _ctx=None):
            return _M_omero.model.Polyline._op_getMarkerEnd.invoke(self, ((), _ctx))

        def begin_getMarkerEnd(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Polyline._op_getMarkerEnd.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getMarkerEnd(self, _r):
            return _M_omero.model.Polyline._op_getMarkerEnd.end(self, _r)

        def setMarkerEnd(self, theMarkerEnd, _ctx=None):
            return _M_omero.model.Polyline._op_setMarkerEnd.invoke(self, ((theMarkerEnd, ), _ctx))

        def begin_setMarkerEnd(self, theMarkerEnd, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Polyline._op_setMarkerEnd.begin(self, ((theMarkerEnd, ), _response, _ex, _sent, _ctx))

        def end_setMarkerEnd(self, _r):
            return _M_omero.model.Polyline._op_setMarkerEnd.end(self, _r)

        def getTextValue(self, _ctx=None):
            return _M_omero.model.Polyline._op_getTextValue.invoke(self, ((), _ctx))

        def begin_getTextValue(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Polyline._op_getTextValue.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getTextValue(self, _r):
            return _M_omero.model.Polyline._op_getTextValue.end(self, _r)

        def setTextValue(self, theTextValue, _ctx=None):
            return _M_omero.model.Polyline._op_setTextValue.invoke(self, ((theTextValue, ), _ctx))

        def begin_setTextValue(self, theTextValue, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Polyline._op_setTextValue.begin(self, ((theTextValue, ), _response, _ex, _sent, _ctx))

        def end_setTextValue(self, _r):
            return _M_omero.model.Polyline._op_setTextValue.end(self, _r)

        def checkedCast(proxy, facetOrCtx=None, _ctx=None):
            return _M_omero.model.PolylinePrx.ice_checkedCast(proxy, '::omero::model::Polyline', facetOrCtx, _ctx)
        checkedCast = staticmethod(checkedCast)

        def uncheckedCast(proxy, facet=None):
            return _M_omero.model.PolylinePrx.ice_uncheckedCast(proxy, facet)
        uncheckedCast = staticmethod(uncheckedCast)

        def ice_staticId():
            return '::omero::model::Polyline'
        ice_staticId = staticmethod(ice_staticId)

    _M_omero.model._t_PolylinePrx = IcePy.defineProxy('::omero::model::Polyline', PolylinePrx)

    _M_omero.model._t_Polyline = IcePy.declareClass('::omero::model::Polyline')

    _M_omero.model._t_Polyline = IcePy.defineClass('::omero::model::Polyline', Polyline, -1, (), True, False, _M_omero.model._t_Shape, (), (
        ('_points', (), _M_omero._t_RString, False, 0),
        ('_markerStart', (), _M_omero._t_RString, False, 0),
        ('_markerEnd', (), _M_omero._t_RString, False, 0),
        ('_textValue', (), _M_omero._t_RString, False, 0)
    ))
    Polyline._ice_type = _M_omero.model._t_Polyline

    Polyline._op_getPoints = IcePy.Operation('getPoints', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero._t_RString, False, 0), ())
    Polyline._op_setPoints = IcePy.Operation('setPoints', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero._t_RString, False, 0),), (), None, ())
    Polyline._op_getMarkerStart = IcePy.Operation('getMarkerStart', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero._t_RString, False, 0), ())
    Polyline._op_setMarkerStart = IcePy.Operation('setMarkerStart', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero._t_RString, False, 0),), (), None, ())
    Polyline._op_getMarkerEnd = IcePy.Operation('getMarkerEnd', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero._t_RString, False, 0), ())
    Polyline._op_setMarkerEnd = IcePy.Operation('setMarkerEnd', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero._t_RString, False, 0),), (), None, ())
    Polyline._op_getTextValue = IcePy.Operation('getTextValue', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero._t_RString, False, 0), ())
    Polyline._op_setTextValue = IcePy.Operation('setTextValue', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero._t_RString, False, 0),), (), None, ())

    _M_omero.model.Polyline = Polyline
    del Polyline

    _M_omero.model.PolylinePrx = PolylinePrx
    del PolylinePrx

# End of module omero.model

__name__ = 'omero'

# End of module omero



"""
   /*
   **   Generated by blitz/resources/templates/combined.vm
   **
   **   Copyright 2007, 2008 Glencoe Software, Inc. All rights reserved.
   **   Use is subject to license terms supplied in LICENSE.txt
   **
   */
"""
try:
   unicode
except NameError:
   # Python 3: "unicode" is built-in
   unicode = str
import Ice
import IceImport
import omero
IceImport.load("omero_model_DetailsI")
IceImport.load("omero_model_FilterSet_ice")
from omero.rtypes import rlong
from collections import namedtuple
_omero = Ice.openModule("omero")
_omero_model = Ice.openModule("omero.model")
__name__ = "omero.model"
class FilterSetI(_omero_model.FilterSet):

      # Property Metadata
      _field_info_data = namedtuple("FieldData", ["wrapper", "nullable"])
      _field_info_type = namedtuple("FieldInfo", [
          "manufacturer",
          "model",
          "lotNumber",
          "serialNumber",
          "instrument",
          "excitationFilterLink",
          "dichroic",
          "emissionFilterLink",
          "details",
      ])
      _field_info = _field_info_type(
          manufacturer=_field_info_data(wrapper=omero.rtypes.rstring, nullable=True),
          model=_field_info_data(wrapper=omero.rtypes.rstring, nullable=True),
          lotNumber=_field_info_data(wrapper=omero.rtypes.rstring, nullable=True),
          serialNumber=_field_info_data(wrapper=omero.rtypes.rstring, nullable=True),
          instrument=_field_info_data(wrapper=omero.proxy_to_instance, nullable=False),
          excitationFilterLink=_field_info_data(wrapper=omero.proxy_to_instance, nullable=True),
          dichroic=_field_info_data(wrapper=omero.proxy_to_instance, nullable=True),
          emissionFilterLink=_field_info_data(wrapper=omero.proxy_to_instance, nullable=True),
          details=_field_info_data(wrapper=omero.proxy_to_instance, nullable=True),
      )  # end _field_info
      MANUFACTURER =  "ome.model.acquisition.FilterSet_manufacturer"
      MODEL =  "ome.model.acquisition.FilterSet_model"
      LOTNUMBER =  "ome.model.acquisition.FilterSet_lotNumber"
      SERIALNUMBER =  "ome.model.acquisition.FilterSet_serialNumber"
      INSTRUMENT =  "ome.model.acquisition.FilterSet_instrument"
      EXCITATIONFILTERLINK =  "ome.model.acquisition.FilterSet_excitationFilterLink"
      DICHROIC =  "ome.model.acquisition.FilterSet_dichroic"
      EMISSIONFILTERLINK =  "ome.model.acquisition.FilterSet_emissionFilterLink"
      DETAILS =  "ome.model.acquisition.FilterSet_details"
      def errorIfUnloaded(self):
          if not self._loaded:
              raise _omero.UnloadedEntityException("Object unloaded:"+str(self))

      def throwNullCollectionException(self,propertyName):
          raise _omero.UnloadedEntityException(""+
          "Error updating collection:" + propertyName +"\n"+
          "Collection is currently null. This can be seen\n" +
          "by testing \""+ propertyName +"Loaded\". This implies\n"+
          "that this collection was unloaded. Please refresh this object\n"+
          "in order to update this collection.\n")

      def _toggleCollectionsLoaded(self, load):
          if load:
              self._excitationFilterLinkSeq = []
              self._excitationFilterLinkLoaded = True;
          else:
              self._excitationFilterLinkSeq = []
              self._excitationFilterLinkLoaded = False;

          if load:
              self._emissionFilterLinkSeq = []
              self._emissionFilterLinkLoaded = True;
          else:
              self._emissionFilterLinkSeq = []
              self._emissionFilterLinkLoaded = False;

          pass

      def __init__(self, id=None, loaded=None):
          super(FilterSetI, self).__init__()
          if id is not None and isinstance(id, (str, unicode)) and ":" in id:
              parts = id.split(":")
              if len(parts) != 2:
                  raise Exception("Invalid proxy string: %s", id)
              if parts[0] != self.__class__.__name__ and \
                 parts[0]+"I" != self.__class__.__name__:
                  raise Exception("Proxy class mismatch: %s<>%s" %
                  (self.__class__.__name__, parts[0]))
              self._id = rlong(parts[1])
              if loaded is None:
                  # If no loadedness was requested with
                  # a proxy string, then assume False.
                  loaded = False
          else:
              # Relying on omero.rtypes.rlong's error-handling
              self._id = rlong(id)
              if loaded is None:
                  loaded = True  # Assume true as previously
          self._loaded = loaded
          if self._loaded:
             self._details = _omero_model.DetailsI()
             self._toggleCollectionsLoaded(True)

      def unload(self, current = None):
          self._loaded = False
          self.unloadManufacturer( )
          self.unloadModel( )
          self.unloadLotNumber( )
          self.unloadSerialNumber( )
          self.unloadInstrument( )
          self.unloadExcitationFilterLink( )
          self.unloadDichroic( )
          self.unloadEmissionFilterLink( )
          self.unloadDetails( )

      def isLoaded(self, current = None):
          return self._loaded
      def unloadCollections(self, current = None):
          self._toggleCollectionsLoaded( False )
      def isGlobal(self, current = None):
          return  False ;
      def isMutable(self, current = None):
          return  True ;
      def isAnnotated(self, current = None):
          return  False ;
      def isLink(self, current = None):
          return  False ;
      def shallowCopy(self, current = None):
            if not self._loaded: return self.proxy()
            copy = FilterSetI()
            copy._id = self._id;
            copy._version = self._version;
            copy._details = None  # Unloading for the moment.
            raise omero.ClientError("NYI")
      def proxy(self, current = None):
          if self._id is None: raise omero.ClientError("Proxies require an id")
          return FilterSetI( self._id.getValue(), False )

      def getDetails(self, current = None):
          self.errorIfUnloaded()
          return self._details

      def unloadDetails(self, current = None):
          self._details = None

      def getId(self, current = None):
          return self._id

      def setId(self, _id, current = None):
          self._id = _id

      def checkUnloadedProperty(self, value, loadedField):
          if value == None:
              self.__dict__[loadedField] = False
          else:
              self.__dict__[loadedField] = True

      def getVersion(self, current = None):
          self.errorIfUnloaded()
          return self._version

      def setVersion(self, version, current = None):
          self.errorIfUnloaded()
          self._version = version

      def unloadManufacturer(self, ):
          self._manufacturerLoaded = False
          self._manufacturer = None;

      def getManufacturer(self, current = None):
          self.errorIfUnloaded()
          return self._manufacturer

      def setManufacturer(self, _manufacturer, current = None, wrap=False):
          self.errorIfUnloaded()
          if wrap and self._field_info.manufacturer.wrapper is not None:
              if _manufacturer is not None:
                  _manufacturer = self._field_info.manufacturer.wrapper(_manufacturer)
          self._manufacturer = _manufacturer
          pass

      def unloadModel(self, ):
          self._modelLoaded = False
          self._model = None;

      def getModel(self, current = None):
          self.errorIfUnloaded()
          return self._model

      def setModel(self, _model, current = None, wrap=False):
          self.errorIfUnloaded()
          if wrap and self._field_info.model.wrapper is not None:
              if _model is not None:
                  _model = self._field_info.model.wrapper(_model)
          self._model = _model
          pass

      def unloadLotNumber(self, ):
          self._lotNumberLoaded = False
          self._lotNumber = None;

      def getLotNumber(self, current = None):
          self.errorIfUnloaded()
          return self._lotNumber

      def setLotNumber(self, _lotNumber, current = None, wrap=False):
          self.errorIfUnloaded()
          if wrap and self._field_info.lotNumber.wrapper is not None:
              if _lotNumber is not None:
                  _lotNumber = self._field_info.lotNumber.wrapper(_lotNumber)
          self._lotNumber = _lotNumber
          pass

      def unloadSerialNumber(self, ):
          self._serialNumberLoaded = False
          self._serialNumber = None;

      def getSerialNumber(self, current = None):
          self.errorIfUnloaded()
          return self._serialNumber

      def setSerialNumber(self, _serialNumber, current = None, wrap=False):
          self.errorIfUnloaded()
          if wrap and self._field_info.serialNumber.wrapper is not None:
              if _serialNumber is not None:
                  _serialNumber = self._field_info.serialNumber.wrapper(_serialNumber)
          self._serialNumber = _serialNumber
          pass

      def unloadInstrument(self, ):
          self._instrumentLoaded = False
          self._instrument = None;

      def getInstrument(self, current = None):
          self.errorIfUnloaded()
          return self._instrument

      def setInstrument(self, _instrument, current = None, wrap=False):
          self.errorIfUnloaded()
          if wrap and self._field_info.instrument.wrapper is not None:
              if _instrument is not None:
                  _instrument = self._field_info.instrument.wrapper(_instrument)
          self._instrument = _instrument
          pass

      def unloadExcitationFilterLink(self, current = None):
          self._excitationFilterLinkLoaded = False
          self._excitationFilterLinkSeq = None;

      def _getExcitationFilterLink(self, current = None):
          self.errorIfUnloaded()
          return self._excitationFilterLinkSeq

      def _setExcitationFilterLink(self, _excitationFilterLink, current = None, wrap=False):
          self.errorIfUnloaded()
          if wrap and self._field_info.excitationFilterLinkSeq.wrapper is not None:
              if _excitationFilterLink is not None:
                  _excitationFilterLink = self._field_info.excitationFilterLinkSeq.wrapper(_excitationFilterLink)
          self._excitationFilterLinkSeq = _excitationFilterLink
          self.checkUnloadedProperty(_excitationFilterLink,'excitationFilterLinkLoaded')

      def isExcitationFilterLinkLoaded(self):
          return self._excitationFilterLinkLoaded

      def sizeOfExcitationFilterLink(self, current = None):
          self.errorIfUnloaded()
          if not self._excitationFilterLinkLoaded: return -1
          return len(self._excitationFilterLinkSeq)

      def copyExcitationFilterLink(self, current = None):
          self.errorIfUnloaded()
          if not self._excitationFilterLinkLoaded: self.throwNullCollectionException("excitationFilterLinkSeq")
          return list(self._excitationFilterLinkSeq)

      def iterateExcitationFilterLink(self):
          self.errorIfUnloaded()
          if not self._excitationFilterLinkLoaded: self.throwNullCollectionException("excitationFilterLinkSeq")
          return iter(self._excitationFilterLinkSeq)

      def addFilterSetExcitationFilterLink(self, target, current = None):
          self.errorIfUnloaded()
          if not self._excitationFilterLinkLoaded: self.throwNullCollectionException("excitationFilterLinkSeq")
          self._excitationFilterLinkSeq.append( target );
          target.setParent( self )

      def addAllFilterSetExcitationFilterLinkSet(self, targets, current = None):
          self.errorIfUnloaded()
          if  not self._excitationFilterLinkLoaded: self.throwNullCollectionException("excitationFilterLinkSeq")
          self._excitationFilterLinkSeq.extend( targets )
          for target in targets:
              target.setParent( self )

      def removeFilterSetExcitationFilterLink(self, target, current = None):
          self.errorIfUnloaded()
          if not self._excitationFilterLinkLoaded: self.throwNullCollectionException("excitationFilterLinkSeq")
          self._excitationFilterLinkSeq.remove( target )
          target.setParent( None )

      def removeAllFilterSetExcitationFilterLinkSet(self, targets, current = None):
          self.errorIfUnloaded()
          if not self._excitationFilterLinkLoaded: self.throwNullCollectionException("excitationFilterLinkSeq")
          for elt in targets:
              elt.setParent( None )
              self._excitationFilterLinkSeq.remove( elt )

      def clearExcitationFilterLink(self, current = None):
          self.errorIfUnloaded()
          if not self._excitationFilterLinkLoaded: self.throwNullCollectionException("excitationFilterLinkSeq")
          for elt in self._excitationFilterLinkSeq:
              elt.setParent( None )
          self._excitationFilterLinkSeq = list()

      def reloadExcitationFilterLink(self, toCopy, current = None):
          self.errorIfUnloaded()
          if self._excitationFilterLinkLoaded:
              raise omero.ClientError("Cannot reload active collection: excitationFilterLinkSeq")
          if not toCopy:
              raise omero.ClientError("Argument cannot be null")
          if toCopy.getId().getValue() != self.getId().getValue():
             raise omero.ClientError("Argument must have the same id as this instance")
          if toCopy.getDetails().getUpdateEvent().getId().getValue() < self.getDetails().getUpdateEvent().getId().getValue():
             raise omero.ClientError("Argument may not be older than this instance")
          copy = toCopy.copyExcitationFilterLink() # May also throw
          for elt in copy:
              elt.setParent( self )
          self._excitationFilterLinkSeq = copy
          toCopy.unloadExcitationFilterLink()
          self._excitationFilterLinkLoaded = True

      def getExcitationFilterLinkCountPerOwner(self, current = None):
          return self._excitationFilterLinkCountPerOwner

      def linkExcitationFilter(self, addition, current = None):
          self.errorIfUnloaded()
          if not self._excitationFilterLinkLoaded: self.throwNullCollectionException("excitationFilterLinkSeq")
          link = _omero_model.FilterSetExcitationFilterLinkI()
          link.link( self, addition );
          self.addFilterSetExcitationFilterLinkToBoth( link, True )
          return link

      def addFilterSetExcitationFilterLinkToBoth(self, link, bothSides):
          self.errorIfUnloaded()
          if not self._excitationFilterLinkLoaded: self.throwNullCollectionException("excitationFilterLinkSeq")
          self._excitationFilterLinkSeq.append( link )
          if bothSides and link.getChild().isLoaded():
              link.getChild().addFilterSetExcitationFilterLinkToBoth( link, False )

      def findFilterSetExcitationFilterLink(self, removal, current = None):
          self.errorIfUnloaded()
          if not self._excitationFilterLinkLoaded: self.throwNullCollectionException("excitationFilterLinkSeq")
          result = list()
          for link in self._excitationFilterLinkSeq:
              if link.getChild() == removal: result.append(link)
          return result

      def unlinkExcitationFilter(self, removal, current = None):
          self.errorIfUnloaded()
          if not self._excitationFilterLinkLoaded: self.throwNullCollectionException("excitationFilterLinkSeq")
          toRemove = self.findFilterSetExcitationFilterLink(removal)
          for next in toRemove:
              self.removeFilterSetExcitationFilterLinkFromBoth( next, True )

      def removeFilterSetExcitationFilterLinkFromBoth(self, link, bothSides, current = None):
          self.errorIfUnloaded()
          if not self._excitationFilterLinkLoaded: self.throwNullCollectionException("excitationFilterLinkSeq")
          self._excitationFilterLinkSeq.remove( link )
          if bothSides and link.getChild().isLoaded():
              link.getChild().removeFilterSetExcitationFilterLinkFromBoth(link, False)

      def linkedExcitationFilterList(self, current = None):
          self.errorIfUnloaded()
          if not self.excitationFilterLinkLoaded: self.throwNullCollectionException("ExcitationFilterLink")
          linked = []
          for link in self._excitationFilterLinkSeq:
              linked.append( link.getChild() )
          return linked

      def unloadDichroic(self, ):
          self._dichroicLoaded = False
          self._dichroic = None;

      def getDichroic(self, current = None):
          self.errorIfUnloaded()
          return self._dichroic

      def setDichroic(self, _dichroic, current = None, wrap=False):
          self.errorIfUnloaded()
          if wrap and self._field_info.dichroic.wrapper is not None:
              if _dichroic is not None:
                  _dichroic = self._field_info.dichroic.wrapper(_dichroic)
          self._dichroic = _dichroic
          pass

      def unloadEmissionFilterLink(self, current = None):
          self._emissionFilterLinkLoaded = False
          self._emissionFilterLinkSeq = None;

      def _getEmissionFilterLink(self, current = None):
          self.errorIfUnloaded()
          return self._emissionFilterLinkSeq

      def _setEmissionFilterLink(self, _emissionFilterLink, current = None, wrap=False):
          self.errorIfUnloaded()
          if wrap and self._field_info.emissionFilterLinkSeq.wrapper is not None:
              if _emissionFilterLink is not None:
                  _emissionFilterLink = self._field_info.emissionFilterLinkSeq.wrapper(_emissionFilterLink)
          self._emissionFilterLinkSeq = _emissionFilterLink
          self.checkUnloadedProperty(_emissionFilterLink,'emissionFilterLinkLoaded')

      def isEmissionFilterLinkLoaded(self):
          return self._emissionFilterLinkLoaded

      def sizeOfEmissionFilterLink(self, current = None):
          self.errorIfUnloaded()
          if not self._emissionFilterLinkLoaded: return -1
          return len(self._emissionFilterLinkSeq)

      def copyEmissionFilterLink(self, current = None):
          self.errorIfUnloaded()
          if not self._emissionFilterLinkLoaded: self.throwNullCollectionException("emissionFilterLinkSeq")
          return list(self._emissionFilterLinkSeq)

      def iterateEmissionFilterLink(self):
          self.errorIfUnloaded()
          if not self._emissionFilterLinkLoaded: self.throwNullCollectionException("emissionFilterLinkSeq")
          return iter(self._emissionFilterLinkSeq)

      def addFilterSetEmissionFilterLink(self, target, current = None):
          self.errorIfUnloaded()
          if not self._emissionFilterLinkLoaded: self.throwNullCollectionException("emissionFilterLinkSeq")
          self._emissionFilterLinkSeq.append( target );
          target.setParent( self )

      def addAllFilterSetEmissionFilterLinkSet(self, targets, current = None):
          self.errorIfUnloaded()
          if  not self._emissionFilterLinkLoaded: self.throwNullCollectionException("emissionFilterLinkSeq")
          self._emissionFilterLinkSeq.extend( targets )
          for target in targets:
              target.setParent( self )

      def removeFilterSetEmissionFilterLink(self, target, current = None):
          self.errorIfUnloaded()
          if not self._emissionFilterLinkLoaded: self.throwNullCollectionException("emissionFilterLinkSeq")
          self._emissionFilterLinkSeq.remove( target )
          target.setParent( None )

      def removeAllFilterSetEmissionFilterLinkSet(self, targets, current = None):
          self.errorIfUnloaded()
          if not self._emissionFilterLinkLoaded: self.throwNullCollectionException("emissionFilterLinkSeq")
          for elt in targets:
              elt.setParent( None )
              self._emissionFilterLinkSeq.remove( elt )

      def clearEmissionFilterLink(self, current = None):
          self.errorIfUnloaded()
          if not self._emissionFilterLinkLoaded: self.throwNullCollectionException("emissionFilterLinkSeq")
          for elt in self._emissionFilterLinkSeq:
              elt.setParent( None )
          self._emissionFilterLinkSeq = list()

      def reloadEmissionFilterLink(self, toCopy, current = None):
          self.errorIfUnloaded()
          if self._emissionFilterLinkLoaded:
              raise omero.ClientError("Cannot reload active collection: emissionFilterLinkSeq")
          if not toCopy:
              raise omero.ClientError("Argument cannot be null")
          if toCopy.getId().getValue() != self.getId().getValue():
             raise omero.ClientError("Argument must have the same id as this instance")
          if toCopy.getDetails().getUpdateEvent().getId().getValue() < self.getDetails().getUpdateEvent().getId().getValue():
             raise omero.ClientError("Argument may not be older than this instance")
          copy = toCopy.copyEmissionFilterLink() # May also throw
          for elt in copy:
              elt.setParent( self )
          self._emissionFilterLinkSeq = copy
          toCopy.unloadEmissionFilterLink()
          self._emissionFilterLinkLoaded = True

      def getEmissionFilterLinkCountPerOwner(self, current = None):
          return self._emissionFilterLinkCountPerOwner

      def linkEmissionFilter(self, addition, current = None):
          self.errorIfUnloaded()
          if not self._emissionFilterLinkLoaded: self.throwNullCollectionException("emissionFilterLinkSeq")
          link = _omero_model.FilterSetEmissionFilterLinkI()
          link.link( self, addition );
          self.addFilterSetEmissionFilterLinkToBoth( link, True )
          return link

      def addFilterSetEmissionFilterLinkToBoth(self, link, bothSides):
          self.errorIfUnloaded()
          if not self._emissionFilterLinkLoaded: self.throwNullCollectionException("emissionFilterLinkSeq")
          self._emissionFilterLinkSeq.append( link )
          if bothSides and link.getChild().isLoaded():
              link.getChild().addFilterSetEmissionFilterLinkToBoth( link, False )

      def findFilterSetEmissionFilterLink(self, removal, current = None):
          self.errorIfUnloaded()
          if not self._emissionFilterLinkLoaded: self.throwNullCollectionException("emissionFilterLinkSeq")
          result = list()
          for link in self._emissionFilterLinkSeq:
              if link.getChild() == removal: result.append(link)
          return result

      def unlinkEmissionFilter(self, removal, current = None):
          self.errorIfUnloaded()
          if not self._emissionFilterLinkLoaded: self.throwNullCollectionException("emissionFilterLinkSeq")
          toRemove = self.findFilterSetEmissionFilterLink(removal)
          for next in toRemove:
              self.removeFilterSetEmissionFilterLinkFromBoth( next, True )

      def removeFilterSetEmissionFilterLinkFromBoth(self, link, bothSides, current = None):
          self.errorIfUnloaded()
          if not self._emissionFilterLinkLoaded: self.throwNullCollectionException("emissionFilterLinkSeq")
          self._emissionFilterLinkSeq.remove( link )
          if bothSides and link.getChild().isLoaded():
              link.getChild().removeFilterSetEmissionFilterLinkFromBoth(link, False)

      def linkedEmissionFilterList(self, current = None):
          self.errorIfUnloaded()
          if not self.emissionFilterLinkLoaded: self.throwNullCollectionException("EmissionFilterLink")
          linked = []
          for link in self._emissionFilterLinkSeq:
              linked.append( link.getChild() )
          return linked


      def ice_postUnmarshal(self):
          """
          Provides additional initialization once all data loaded
          """
          pass # Currently unused


      def ice_preMarshal(self):
          """
          Provides additional validation before data is sent
          """
          pass # Currently unused

      def __getattr__(self, name):
          """
          Reroutes all access to object.field through object.getField() or object.isField()
          """
          if "_" in name:  # Ice disallows underscores, so these should be treated normally.
              return object.__getattribute__(self, name)
          field  = "_" + name
          capitalized = name[0].capitalize() + name[1:]
          getter = "get" + capitalized
          questn = "is" + capitalized
          try:
              self.__dict__[field]
              if hasattr(self, getter):
                  method = getattr(self, getter)
                  return method()
              elif hasattr(self, questn):
                  method = getattr(self, questn)
                  return method()
          except:
              pass
          raise AttributeError("'%s' object has no attribute '%s' or '%s'" % (self.__class__.__name__, getter, questn))

      def __setattr__(self, name, value):
          """
          Reroutes all access to object.field through object.getField(), with the caveat
          that all sets on variables starting with "_" are permitted directly.
          """
          if name.startswith("_"):
              self.__dict__[name] = value
              return
          else:
              field  = "_" + name
              setter = "set" + name[0].capitalize() + name[1:]
              if hasattr(self, field) and hasattr(self, setter):
                  method = getattr(self, setter)
                  return method(value)
          raise AttributeError("'%s' object has no attribute '%s'" % (self.__class__.__name__, setter))

_omero_model.FilterSetI = FilterSetI



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
...
"""
from __future__ import division
from __future__ import print_function

#
#  Copyright (C) 2009 University of Dundee. All rights reserved.
#
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License along
#  with this program; if not, write to the Free Software Foundation, Inc.,
#  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#


from future import standard_library
standard_library.install_aliases()
from builtins import str
from builtins import range
from past.utils import old_div
from builtins import object
import tempfile
import logging
import time
import sys
import csv
import re
from threading import Thread
from getpass import getpass
from getopt import getopt, GetoptError
from queue import Queue

from omero.rtypes import rdouble, rstring, rint, unwrap
from omero.model import OriginalFileI, PlateI, PlateAnnotationLinkI, ImageI, \
    FileAnnotationI, RoiI, EllipseI, PointI
from omero.grid import ImageColumn, WellColumn, RoiColumn, LongColumn, \
    DoubleColumn

from omero.sys import ParametersI
from omero.util.temp_files import create_path
from omero import client

from xml.etree.cElementTree import ElementTree, iterparse

log = logging.getLogger("omero.util.populate_roi")


def usage(error):
    """Prints usage so that we don't have to. :)"""
    cmd = sys.argv[0]
    print("""%s
Usage: %s [-s hostname] [-u username | -k session_key] <-p port> [plate_id]
Runs measurement population code for a given plate.

Options:
  -s    OMERO hostname to use
  -p    OMERO port to use [defaults to 4064]
  -u    OMERO username to use
  -k    OMERO session key to use
  -m    Measurement index to populate
  -i    Dump measurement information and exit (no population)
  -d    Print debug statements
  -t    Number of threads to use when populating [defaults to 1]

Examples:
  %s -s localhost -p 4063 -u bob 27

Report bugs to ome-devel@lists.openmicroscopy.org.uk""" % (error, cmd, cmd))
    sys.exit(2)

###
# Worker and ThreadPool from...
# http://code.activestate.com/recipes/577187-python-thread-pool/
###


class Worker(Thread):

    """Thread executing tasks from a given tasks queue"""

    def __init__(self, tasks):
        Thread.__init__(self)
        self.tasks = tasks
        self.daemon = True
        self.start()

    def run(self):
        while True:
            func, args, kargs = self.tasks.get()
            try:
                func(*args, **kargs)
            except Exception as e:
                log.exception(e)
            self.tasks.task_done()


class ThreadPool(object):

    """Pool of threads consuming tasks from a queue"""

    def __init__(self, num_threads):
        self.tasks = Queue(num_threads)
        for _ in range(num_threads):
            Worker(self.tasks)

    def add_task(self, func, *args, **kargs):
        """Add a task to the queue"""
        self.tasks.put((func, args, kargs))

    def wait_completion(self):
        """Wait for completion of all the tasks in the queue"""
        self.tasks.join()

# Global thread pool for use by ROI workers
thread_pool = None


def get_thread_pool():
    global thread_pool
    if thread_pool is None:
        thread_pool = ThreadPool(1)
    return thread_pool


class MeasurementError(Exception):

    """
    Raised by the analysis or measurement context when an error condition
    is reached.
    """
    pass


class DownloadingOriginalFileProvider(object):

    """
    Provides original file data by downloading it from an OMERO raw file store.
    """

    # Default raw file store buffer size
    BUFFER_SIZE = 1024 * 1024  # 1MB

    def __init__(self, service_factory):
        self.service_factory = service_factory
        self.raw_file_store = self.service_factory.createRawFileStore()
        self.dir = create_path("populate_roi", "dir", folder=True)

    def get_original_file_data(self, original_file):
        """
        Downloads an original file to a temporary file and returns an open
        file handle to that temporary file seeked to zero. The caller is
        responsible for closing the temporary file.
        """
        log.info("Downloading original file: %d" % original_file.id.val)
        self.raw_file_store.setFileId(original_file.id.val)
        temporary_file = tempfile.NamedTemporaryFile(mode='rt+',
                                                     dir=str(self.dir))
        size = original_file.size.val
        for i in range((old_div(size, self.BUFFER_SIZE)) + 1):
            index = i * self.BUFFER_SIZE
            data = self.raw_file_store.read(index, self.BUFFER_SIZE)
            temporary_file.write(data.decode("utf-8"))
        temporary_file.seek(0)
        temporary_file.truncate(size)
        return temporary_file

    def __delete__(self):
        self.raw_file_store.close()


class AbstractPlateAnalysisCtx(object):

    """
    Abstract class which aggregates and represents all measurement runs made on
    a given Plate.
    """

    DEFAULT_ORIGINAL_FILE_PROVIDER = DownloadingOriginalFileProvider

    def __init__(self, images, original_files, original_file_image_map,
                 plate_id, service_factory):
        super(AbstractPlateAnalysisCtx, self).__init__()
        self.images = images
        self.numcols, self.numrows = self.guess_geometry(self.images)
        self.original_files = original_files
        self.original_file_image_map = original_file_image_map
        self.plate_id = plate_id
        self.service_factory = service_factory
        self.log_files = dict()
        self.detail_files = dict()
        self.measurements = dict()

    def guess_geometry(self, images):
        max_col = 0
        max_row = 0
        for image in images:
            # Using only first well sample link
            ws = image.copyWellSamples()[0]
            well = ws.well
            max_col = max(max_col, well.column.val)
            max_row = max(max_row, well.row.val)
        return (max_col + 1, max_row + 1)

    def colrow_from_wellnumber(self, width, wellnumber):
        x = wellnumber - 1
        col = x % width
        row = old_div(x, width)
        return (col, row)

    def image_from_wellnumber(self, wellnumber):
        col, row = self.colrow_from_wellnumber(self.numcols, wellnumber)
        log.debug("Finding image for %s (%s,%s)..." % (wellnumber, col, row))
        for image in self.images:
            well = image.copyWellSamples()[0].well
            if well.column.val == col and well.row.val == row:
                return image
        raise Exception(
            "Could not find image for (col,row)==(%s,%s)" % (col, row))

    ###
    # Abstract methods
    ###

    def is_this_type(klass):
        """
        Concrete implementations are to return True if the class pertinent
        for the original files associated with the plate.
        """
        raise Exception("To be implemented by concrete implementations.")
    is_this_type = classmethod(is_this_type)

    def get_measurement_count(self):
        """Returns the number of recognized measurement runs."""
        raise Exception("To be implemented by concrete implementations.")

    def get_measurement_ctx(self, index):
        """Returns the measurement context for a given index."""
        raise Exception("To be implemented by concrete implementations.")

    def get_result_file_count(self, measurement_index):
        """
        Return the number of result files associated with a measurement run.
        """
        raise Exception("To be implemented by concrete implementations.")


class MIASPlateAnalysisCtx(AbstractPlateAnalysisCtx):

    """
    MIAS dataset concrete class implementation of an analysis context. MIAS
    measurements are aggregated based on a single "log" file. A result
    file is present for each stitched (of multiple fields) mosaic and
    contains the actual measured results and ROI.
    """

    # Python datetime format string of the log filename completion date/time
    datetime_format = '%Y-%m-%d-%Hh%Mm%Ss'

    # Regular expression matching a log filename
    log_regex = re.compile(r'.*log(\d+-\d+-\d+-\d+h\d+m\d+s).txt$')

    # Regular expression matching a result filename
    detail_regex = re.compile(
        r'^Well(\d+)_(.*)_detail_(\d+-\d+-\d+-\d+h\d+m\d+s).txt$')

    # Companion file format
    companion_format = 'Companion/MIAS'

    def __init__(self, images, original_files, original_file_image_map,
                 plate_id, service_factory):
        super(MIASPlateAnalysisCtx, self).__init__(
            images, original_files, original_file_image_map, plate_id,
            service_factory)
        self._populate_log_and_detail_files()
        self._populate_measurements()

    def _populate_log_and_detail_files(self):
        """
        Strips out erroneous files and collects the log and result original
        files based on regular expression matching.
        """
        for original_file in self.original_files:
            if original_file.mimetype and \
                    original_file.mimetype.val != self.companion_format:
                # In OMERO5, the mimetype will not be set.
                continue
            name = original_file.name.val
            match = self.log_regex.match(name)
            if match:
                d = time.strptime(match.group(1), self.datetime_format)
                self.log_files[d] = original_file
                continue
            match = self.detail_regex.match(name)
            if match:
                d = time.strptime(match.group(3), self.datetime_format)
                self.detail_files[d] = original_file
                continue

    def _populate_measurements(self):
        """
        Result original files are only recognizable as part of a given
        measurement (declared by a log file) based upon their parsed
        date/time of completion as encoded in the filename. This method
        collects result original files and groups them by collective
        parsed date/time of completion.
        """
        log_timestamps = list(self.log_files.keys())
        log_timestamps.sort()
        detail_timestamps = list(self.detail_files.keys())
        detail_timestamps.sort()
        for log_timestamp in log_timestamps:
            self.measurements[log_timestamp] = list()
        for detail_timestamp in detail_timestamps:
            for log_timestamp in log_timestamps:
                if detail_timestamp < log_timestamp:
                    self.measurements[log_timestamp].append(
                        self.detail_files[detail_timestamp])
                    break

    ###
    # Abstract method implementations
    ###

    def is_this_type(klass, original_files):
        for original_file in original_files:
            format = unwrap(original_file.mimetype)
            # In OMERO5, "Companion/*" is unused.
            if (format is None or format == klass.companion_format) \
               and klass.log_regex.match(original_file.name.val):
                return True
    is_this_type = classmethod(is_this_type)

    def get_measurement_count(self):
        return len(list(self.measurements.keys()))

    def get_measurement_ctx(self, index):
        key = list(self.log_files.keys())[index]
        sf = self.service_factory
        original_file = self.log_files[key]
        result_files = self.measurements[key]
        provider = self.DEFAULT_ORIGINAL_FILE_PROVIDER(sf)
        return MIASMeasurementCtx(self, sf, provider, original_file,
                                  result_files)

    def get_result_file_count(self, measurement_index):
        key = list(self.log_files.keys())[measurement_index]
        return len(self.measurements[key])


class FlexPlateAnalysisCtx(AbstractPlateAnalysisCtx):

    """
    Flex dataset concrete class implementation of an analysis context. Flex
    measurements are aggregated in a single ".res" XML file and contain no
    ROI.
    """

    # Companion file format
    companion_format = 'Companion/Flex'

    def __init__(self, images, original_files, original_file_image_map,
                 plate_id, service_factory):
        super(FlexPlateAnalysisCtx, self).__init__(
            images, original_files, original_file_image_map, plate_id,
            service_factory)
        path_original_file_map = dict()
        for original_file in original_files:
            path = original_file.path.val
            name = original_file.name.val
            format = original_file.mimetype.val
            if format == self.companion_format and name.endswith('.res'):
                path_original_file_map[path] = original_file
        self.measurements = list(path_original_file_map.values())

    ###
    # Abstract method implementations
    ###

    def is_this_type(klass, original_files):
        for original_file in original_files:
            format = unwrap(original_file.mimetype)
            name = original_file.name.val
            if format == klass.companion_format and name.endswith('.res'):
                return True
        return False
    is_this_type = classmethod(is_this_type)

    def get_measurement_count(self):
        return len(self.measurements)

    def get_measurement_ctx(self, index):
        sf = self.service_factory
        original_file = self.measurements[index]
        result_files = []
        provider = self.DEFAULT_ORIGINAL_FILE_PROVIDER(sf)
        return FlexMeasurementCtx(self, sf, provider, original_file,
                                  result_files)

    def get_result_file_count(self, measurement_index):
        return 1


class InCellPlateAnalysisCtx(AbstractPlateAnalysisCtx):

    """
    InCell dataset concrete class implementation of an analysis context.
    InCell measurements are from InCell Analyzer and are aggregated in a
    single gargantuan (often larger than 100MB per plate) XML file.
    """

    # Companion file format
    companion_format = 'Companion/InCell'

    def __init__(self, images, original_files, original_file_image_map,
                 plate_id, service_factory):
        super(InCellPlateAnalysisCtx, self).__init__(
            images, original_files, original_file_image_map, plate_id,
            service_factory)
        path_original_file_map = dict()
        for original_file in original_files:
            path = original_file.path.val
            name = original_file.name.val
            format = original_file.mimetype.val
            if format == self.companion_format and name.endswith('.xml'):
                path_original_file_map[path] = original_file
        self.measurements = list(path_original_file_map.values())

    ###
    # Abstract method implementations
    ###

    def is_this_type(klass, original_files):
        for original_file in original_files:
            format = unwrap(original_file.mimetype)
            name = original_file.name.val
            if format == klass.companion_format and name.endswith('.xml'):
                return True
        return False
    is_this_type = classmethod(is_this_type)

    def get_measurement_count(self):
        return len(self.measurements)

    def get_measurement_ctx(self, index):
        sf = self.service_factory
        original_file = self.measurements[index]
        result_files = []
        provider = self.DEFAULT_ORIGINAL_FILE_PROVIDER(sf)
        return InCellMeasurementCtx(self, sf, provider, original_file,
                                    result_files)

    def get_result_file_count(self, measurement_index):
        return 1


class PlateAnalysisCtxFactory(object):

    """
    The plate analysis context factory is responsible for detecting and
    returning a plate analysis context instance for a given plate.
    """

    implementations = (FlexPlateAnalysisCtx, MIASPlateAnalysisCtx,
                       InCellPlateAnalysisCtx)

    def __init__(self, service_factory):
        self.service_factory = service_factory
        self.query_service = self.service_factory.getQueryService()

    def find_images_for_plate(self, plate_id):
        """
        Retrieves all the images associated with a given plate. Fetched
        are the Image's WellSample, the WellSample's Well, the annotation
        stack associated with the Image and each annotation's linked
        original file.
        """
        # The query that follows is doublely linked:
        #  * Image --> WellSample --> Well
        #  * Well --> WellSample --> Image
        # This is to facilitate later "ordered" access of fields/well
        # samples required by certain measurement contexts (notably InCell).
        log.debug("Loading image...")
        images = self.query_service.findAllByQuery(
            'select img from Image as img '
            'join fetch img.wellSamples as ws '
            'join fetch ws.well as w '
            'join fetch w.wellSamples as ws2 '
            'join w.plate as p '
            'left outer join fetch img.annotationLinks as ia_links '
            'left outer join fetch ia_links.child as ia '
            'left outer join fetch ia.file as i_o_file '
            'where p.id = %d' % plate_id, None)
        log.debug("Loading plate...")
        plate = self.query_service.findByQuery(
            'select p from Plate p '
            'left outer join fetch p.annotationLinks as pa_links '
            'left outer join fetch pa_links.child as pa '
            'left outer join fetch pa.file as p_o_file '
            'where p.id = %d' % plate_id, None)
        log.debug("Linking plate and images...")
        for image in images:
            for ws in image.copyWellSamples():
                ws.well.plate = plate
        return images, plate

    def gather_original_files(self, obj, original_files,
                              original_file_obj_map):
        for annotation_link in obj.copyAnnotationLinks():
            annotation = annotation_link.child
            if isinstance(annotation, FileAnnotationI):
                f = annotation.file
                original_files.add(f)
                if original_file_obj_map is not None:
                    original_file_obj_map[f.id.val] = obj

    # OMERO5 support

    def find_filesets_for_plate(self, plateid):
        """
        OMERO5 support. See #12235
        """
        return self.query_service.findAllByQuery((
            'select ofile from Fileset f '
            'join f.usedFiles as fse '
            'join fse.originalFile as ofile '
            'join f.images as i '
            'join i.wellSamples ws '
            'join ws.well w '
            'join w.plate p '
            'where p.id = :id'),
            ParametersI().addId(plateid))

    def get_analysis_ctx(self, plate_id):
        """Retrieves a plate analysis context for a given plate."""
        # Using a set since 1) no one was using the image.id key and 2)
        # we are now also collecting original files from plates (MIAS)
        # for which there's no clear key. Since all the files are loaded
        # in a single shot, double linking should not cause a problem.
        original_files = set()
        original_file_image_map = dict()
        images, plate = self.find_images_for_plate(plate_id)
        fileset = self.find_filesets_for_plate(plate_id)
        if fileset:
            original_files.update(fileset)
            self.gather_original_files(plate, original_files, None)
        else:
            plates = set()
            for image in images:
                for ws in image.copyWellSamples():
                    plate = ws.well.plate
                    if plate not in plates:
                        plates.add(plate)
                        self.gather_original_files(plate, original_files, None)
                self.gather_original_files(
                    image, original_files, original_file_image_map)
        for klass in self.implementations:
            if klass.is_this_type(original_files):
                return klass(images, original_files,
                             original_file_image_map,
                             plate_id, self.service_factory)
        raise MeasurementError(
            "Unable to find suitable analysis context for plate: %d" %
            plate_id)


class MeasurementParsingResult(object):

    """
    Holds the results of a measurement parsing event.
    """

    def __init__(self, sets_of_columns=None):
        if sets_of_columns is None:
            self.sets_of_columns = list()
        else:
            self.sets_of_columns = sets_of_columns

    def append_columns(self, columns):
        """Adds a set of columns to the parsing result."""
        self.sets_of_columns.append(columns)


class AbstractMeasurementCtx(object):

    """
    Abstract class which aggregates and represents all the results produced
    from a given measurement run. It also provides a scaffold for interacting
    with the OmeroTables infrastructure.
    """

    # The number of ROI to have parsed before streaming them to the server
    ROI_UPDATE_LIMIT = 1000

    def __init__(self, analysis_ctx, service_factory, original_file_provider,
                 original_file, result_files):
        super(AbstractMeasurementCtx, self).__init__()
        self.thread_pool = get_thread_pool()
        self.analysis_ctx = analysis_ctx
        self.service_factory = service_factory
        self.original_file_provider = original_file_provider
        self.query_service = self.service_factory.getQueryService()
        self.update_service = self.service_factory.getUpdateService()
        self.original_file = original_file
        self.result_files = result_files

        # Establish the rest of our initial state
        self.wellimages = dict()
        for image in self.analysis_ctx.images:
            for well_sample in image.copyWellSamples():
                well = well_sample.well
                idx = well.copyWellSamples().index(well_sample)
                row = well.row.val
                column = well.column.val
                if row not in self.wellimages:
                    self.wellimages[row] = dict()
                if column not in self.wellimages[row]:
                    self.wellimages[row][column] = []
                # Now we save the image at it's proper index
                l = self.wellimages[row][column]
                for x in range(idx - len(l) + 1):
                    l.append(None)
                l[idx] = image

    def get_well_images(self, row, col):
        """
        Takes a row and a col index and returns a tuple
        of Well and image. Either might be None. Uses the
        first image found to find the Well and therefore
        must be loaded (image->wellSample->well)
        """
        try:
            images = self.wellimages[row][col]
            if not images:
                return (None, None)
            image = images[0]
            well = image.copyWellSamples()[0].well
            return (well, images)
        except KeyError:
            # This has the potential to happen alot with the
            # datasets we have given the split machine acquisition
            # ".flex" file storage.
            log.warn("WARNING: Missing data for row %d column %d" %
                     (row, col))
            return (None, None)

    def update_table(self, columns):
        """Updates the OmeroTables instance backing our results."""
        # Create a new OMERO table to store our measurement results
        sr = self.service_factory.sharedResources()
        name = self.get_name()
        self.table = sr.newTable(1, '/%s.r5' % name)
        if self.table is None:
            raise MeasurementError(
                "Unable to create table: %s" % name)

        # Retrieve the original file corresponding to the table for the
        # measurement, link it to the file annotation representing the
        # umbrella measurement run, link the annotation to the plate from
        # which it belongs and save the file annotation.
        table_original_file = self.table.getOriginalFile()
        table_original_file_id = table_original_file.id.val
        log.info("Created new table: %d" % table_original_file_id)
        unloaded_o_file = OriginalFileI(table_original_file_id, False)
        self.file_annotation.file = unloaded_o_file
        unloaded_plate = PlateI(self.analysis_ctx.plate_id, False)
        plate_annotation_link = PlateAnnotationLinkI()
        plate_annotation_link.parent = unloaded_plate
        plate_annotation_link.child = self.file_annotation
        plate_annotation_link = \
            self.update_service.saveAndReturnObject(plate_annotation_link)
        self.file_annotation = plate_annotation_link.child

        t0 = int(time.time() * 1000)
        self.table.initialize(columns)
        log.debug("Table init took %sms" % (int(time.time() * 1000) - t0))
        t0 = int(time.time() * 1000)
        column_report = dict()
        for column in columns:
            column_report[column.name] = len(column.values)
        log.debug("Column report: %r" % column_report)
        self.table.addData(columns)
        self.table.close()
        log.info("Table update took %sms" % (int(time.time() * 1000) - t0))

    def create_file_annotation(self, set_of_columns):
        """
        Creates a file annotation to represent a set of columns from our
        measurment.
        """
        self.file_annotation = FileAnnotationI()
        self.file_annotation.ns = \
            rstring('openmicroscopy.org/omero/measurement')
        name = self.get_name(set_of_columns)
        self.file_annotation.description = rstring(name)

    def update_rois(self, rois, batches, batch_no):
        """
        Updates a set of ROI for a given batch updating the batches
        dictionary with the saved IDs.
        """
        log.debug("Saving %d ROI for batch %d" % (len(rois), batch_no))
        t0 = int(time.time() * 1000)
        roi_ids = self.update_service.saveAndReturnIds(rois)
        log.info("Batch %d ROI update took %sms" %
                 (batch_no, int(time.time() * 1000) - t0))
        batches[batch_no] = roi_ids

    def image_from_original_file(self, original_file):
        """Returns the image from which an original file has originated."""
        m = self.analysis_ctx.original_file_image_map
        return m[original_file.id.val]

    def parse_and_populate(self):
        """
        Calls parse and populate, updating the OmeroTables instance backing
        our results and the OMERO database itself.
        """
        result = self.parse()
        if result is None:
            return
        for i, columns in enumerate(result.sets_of_columns):
            self.create_file_annotation(i)
            self.parse_and_populate_roi(columns)
            self.populate(columns)

    ###
    # Abstract methods
    ###

    def get_name(self, set_of_columns=None):
        """Returns the name of the measurement, and a set of columns."""
        raise Exception("To be implemented by concrete implementations.")

    def parse(self):
        """Parses result files, returning a MeasurementParsingResult."""
        raise Exception("To be implemented by concrete implementations.")

    def parse_and_populate_roi(self, columns):
        """
        Parses and populates ROI from column data in the OMERO database.
        """
        raise Exception("To be implemented by concrete implementations.")

    def populate(self, columns):
        """
        Populates an OmeroTables instance backing our results and ROI
        linkages.
        """
        raise Exception("To be implemented by concrete implementations.")


class MIASMeasurementCtx(AbstractMeasurementCtx):

    """
    MIAS measurements are a set of tab delimited text files per well. Each
    TSV file's content is prefixed by the analysis parameters.
    """

    # The OmeroTable ImageColumn index
    IMAGE_COL = 0

    # The OmeroTable RoiColumn index
    ROI_COL = 1

    # Expected columns in NEO datasets
    NEO_EXPECTED = ('Image', 'ROI', 'Label', 'Row', 'Col', 'Nucleus Area',
                    'Cell Diam.', 'Cell Type', 'Mean Nucleus Intens.')

    # Expected columns in MNU datasets
    MNU_EXPECTED = ('Image', 'ROI', 'row', 'col', 'type')

    def __init__(self, analysis_ctx, service_factory, original_file_provider,
                 original_file, result_files):
        super(MIASMeasurementCtx, self).__init__(
            analysis_ctx, service_factory, original_file_provider,
            original_file, result_files)

    def get_empty_columns(self, n_columns):
        """
        Retrieves a set of empty OmeroTables columns for the analysis results
        prefixed by an ImageColumn and RoiColumn to handle these linked
        object indexes.
        """
        columns = [ImageColumn('Image', '', list()),
                   RoiColumn('ROI', '', list())]
        for i in range(n_columns):
            columns.append(DoubleColumn('', '', list()))
        return columns

    ###
    # Overriding abstract implementation
    ###

    def image_from_original_file(self, original_file):
        """
        Overriding the abstract implementation since the companion
        files are no longer attached to the images, but only to the plate
        for MIAS. Instead, we use the filename itself to find the image.
        """
        name = original_file.name.val
        # Copy: '^Well(\d+)_(.*)_detail_(\d+-\d+-\d+-\d+h\d+m\d+s).txt$'
        match = MIASPlateAnalysisCtx.detail_regex.match(name)
        if match:
            well_num = int(match.group(1))
            return self.analysis_ctx.image_from_wellnumber(well_num)
        else:
            raise Exception("Not a detail file")

    ###
    # Abstract method implementations
    ###

    def get_name(self, set_of_columns=None):
        return self.original_file.name.val[:-4]

    def parse(self):
        columns = None
        for result_file in self.result_files:
            log.info("Parsing: %s" % result_file.name.val)
            image = self.image_from_original_file(result_file)
            provider = self.original_file_provider
            data = provider.get_original_file_data(result_file)
            try:
                rows = list(csv.reader(data, delimiter='\t'))
            finally:
                data.close()
            rows.reverse()
            if columns is None:
                columns = self.get_empty_columns(len(rows[0]))
            for row in rows:
                try:
                    for i, value in enumerate(row):
                        value = float(value)
                        columns[i + 2].values.append(value)
                    columns[self.IMAGE_COL].values.append(image.id.val)
                except ValueError:
                    for i, value in enumerate(row):
                        columns[i + 2].name = value
                    break
        log.debug("Returning %d columns" % len(columns))
        return MeasurementParsingResult([columns])

    def _parse_neo_roi(self, columns):
        """Parses out ROI from OmeroTables columns for 'NEO' datasets."""
        log.debug("Parsing %s NEO ROIs..." % (len(columns[0].values)))
        image_ids = columns[self.IMAGE_COL].values
        rois = list()
        # Save our file annotation to the database so we can use an unloaded
        # annotation for the saveAndReturnIds that will be triggered below.
        self.file_annotation = \
            self.update_service.saveAndReturnObject(self.file_annotation)
        unloaded_file_annotation = \
            FileAnnotationI(self.file_annotation.id.val, False)
        batch_no = 1
        batches = dict()
        for i, image_id in enumerate(image_ids):
            unloaded_image = ImageI(image_id, False)
            roi = RoiI()
            shape = EllipseI()
            values = columns[6].values
            diameter = rdouble(float(values[i]))
            shape.theZ = rint(0)
            shape.theT = rint(0)
            values = columns[4].values
            shape.x = rdouble(float(values[i]))
            values = columns[3].values
            shape.y = rdouble(float(values[i]))
            shape.radiusX = diameter
            shape.radiusY = diameter
            roi.addShape(shape)
            roi.image = unloaded_image
            roi.linkAnnotation(unloaded_file_annotation)
            rois.append(roi)
            if len(rois) == self.ROI_UPDATE_LIMIT:
                self.thread_pool.add_task(
                    self.update_rois, rois, batches, batch_no)
                rois = list()
                batch_no += 1
        self.thread_pool.add_task(self.update_rois, rois, batches, batch_no)
        self.thread_pool.wait_completion()
        batch_keys = list(batches.keys())
        batch_keys.sort()
        for k in batch_keys:
            columns[self.ROI_COL].values += batches[k]

    def _parse_mnu_roi(self, columns):
        """Parses out ROI from OmeroTables columns for 'MNU' datasets."""
        log.debug("Parsing %s MNU ROIs..." % (len(columns[0].values)))
        image_ids = columns[self.IMAGE_COL].values
        rois = list()
        # Save our file annotation to the database so we can use an unloaded
        # annotation for the saveAndReturnIds that will be triggered below.
        self.file_annotation = \
            self.update_service.saveAndReturnObject(self.file_annotation)
        unloaded_file_annotation = \
            FileAnnotationI(self.file_annotation.id.val, False)
        batch_no = 1
        batches = dict()
        for i, image_id in enumerate(image_ids):
            unloaded_image = ImageI(image_id, False)
            roi = RoiI()
            shape = PointI()
            shape.theZ = rint(0)
            shape.theT = rint(0)
            values = columns[3].values
            shape.x = rdouble(float(values[i]))
            values = columns[2].values
            shape.y = rdouble(float(values[i]))
            roi.addShape(shape)
            roi.image = unloaded_image
            roi.linkAnnotation(unloaded_file_annotation)
            rois.append(roi)
            if len(rois) == self.ROI_UPDATE_LIMIT:
                self.thread_pool.add_task(
                    self.update_rois, rois, batches, batch_no)
                rois = list()
                batch_no += 1
        self.thread_pool.add_task(self.update_rois, rois, batches, batch_no)
        self.thread_pool.wait_completion()
        batch_keys = list(batches.keys())
        batch_keys.sort()
        for k in batch_keys:
            columns[self.ROI_COL].values += batches[k]

    def parse_and_populate_roi(self, columns):
        names = [column.name for column in columns]
        neo = [name in self.NEO_EXPECTED for name in names]
        mnu = [name in self.MNU_EXPECTED for name in names]
        for name in names:
            log.debug("Column: %s" % name)
        if len(columns) == 9 and False not in neo:
            self._parse_neo_roi(columns)
        elif len(columns) == 5 and False not in mnu:
            self._parse_mnu_roi(columns)
        else:
            log.warn("Unknown ROI type for MIAS dataset: %r" % names)

    def populate(self, columns):
        """
        Query performed::

                first_roi = columns[self.ROI_COL].values[0]
                first_roi = self.query_service.findByQuery(
                        'select roi from Roi as roi ' \
                        'join fetch roi.annotationLinks as link ' \
                        'join fetch link.child ' \
                        'where roi.id = %d' % first_roi, None)
                self.file_annotation = first_roi.copyAnnotationLinks()[0].child
        """
        self.update_table(columns)


class FlexMeasurementCtx(AbstractMeasurementCtx):

    """
    Flex measurements are located deep within a ".res" XML file container
    and contain no ROI.
    """

    # The XPath to the <Area> which aggregate an acquisition
    AREA_XPATH = './/Areas/Area'

    # The XPath to the an analysis <Parameter>; will become a column header
    # and is below AREA_XPATH
    PARAMETER_XPATH = './/Wells/ResultParameters/Parameter'

    # The XPath to a <Well> which has had at least one acquisition event
    # within and is below AREA_XPATH
    WELL_XPATH = './/Wells/Well'

    # The XPath to a <Result> for a given well and is below WELL_XPATH
    RESULT_XPATH = './/Result'

    def __init__(self, analysis_ctx, service_factory, original_file_provider,
                 original_file, result_files):
        super(FlexMeasurementCtx, self).__init__(
            analysis_ctx, service_factory, original_file_provider,
            original_file, result_files)

    def get_empty_columns(self, headers):
        """
        Retrieves a set of empty OmeroTables columns for the analysis results
        prefixed by a WellColumn to handle linked object indexes.
        """
        columns = {'Well': WellColumn('Well', '', list())}
        for header in headers:
            columns[header] = DoubleColumn(header, '', list())
        return columns

    ###
    # Abstract method implementations
    ###

    def get_name(self, set_of_columns=None):
        return self.original_file.name.val[:-4]

    def parse(self):
        log.info("Parsing: %s" % self.original_file.name.val)
        provider = self.original_file_provider
        data = provider.get_original_file_data(self.original_file)
        try:
            et = ElementTree(file=data)
        finally:
            data.close()
        root = et.getroot()
        areas = root.findall(self.AREA_XPATH)
        log.debug("Area count: %d" % len(areas))
        for i, area in enumerate(areas):
            result_parameters = area.findall(self.PARAMETER_XPATH)
            log.debug("Area %d result children: %d" %
                      (i, len(result_parameters)))
            if len(result_parameters) == 0:
                log.warn("%s contains no analysis data." % self.get_name())
                return
            headers = list()
            for result_parameter in result_parameters:
                headers.append(result_parameter.text)
            columns = self.get_empty_columns(headers)
            wells = area.findall(self.WELL_XPATH)
            for well in wells:
                # Rows and columns are 1-indexed, OMERO wells are 0-indexed
                row = int(well.get('row')) - 1
                column = int(well.get('col')) - 1
                try:
                    v = columns['Well'].values
                    wellobj, images = self.get_well_images(row, column)
                    if not wellobj:
                        continue
                    v.append(wellobj.id.val)
                except:
                    log.exception("ERROR: Failed to get well images")
                    continue
                results = well.findall(self.RESULT_XPATH)
                for result in results:
                    name = result.get('name')
                    columns[name].values.append(float(result.text))
        return MeasurementParsingResult([list(columns.values())])

    def parse_and_populate_roi(self, columns):
        pass

    def populate(self, columns):
        self.update_table(columns)


class InCellMeasurementCtx(AbstractMeasurementCtx):

    """
    InCell Analyzer measurements are located deep within an XML file container.
    """

    # Cells expected centre of gravity columns
    CELLS_CG_EXPECTED = ['Cell: cgX', 'Cell: cgY']

    # Nulcei expected centre of gravity columns
    NUCLEI_CG_EXPECTED = ['Nucleus: cgX', 'Nucleus: cgY']

    # Expected source attribute value for cell data
    CELLS_SOURCE = 'Cells'

    # Expected source attribute value for nuclei data
    NUCLEI_SOURCE = 'Nuclei'

    # Expected source attribute value for organelle data
    ORGANELLES_SOURCE = 'Organelles'

    def __init__(self, analysis_ctx, service_factory, original_file_provider,
                 original_file, result_files):
        super(InCellMeasurementCtx, self).__init__(
            analysis_ctx, service_factory, original_file_provider,
            original_file, result_files)

    def check_sparse_data(self, columns):
        """
        Checks a set of columns for sparse data (one column shorter than
        the rest) and adds -1 where appropriate.
        """
        length = None
        for i, column in enumerate(columns):
            if column.name == 'ROI':
                # ROI are processed late so we don't care if this column
                # is sparse or not.
                continue
            current_length = len(column.values)
            if length is not None:
                if current_length > length:
                    log.debug("%s length %d > %d modding previous column" %
                              (column.name, current_length, length))
                    columns[i - 1].values.append(-1.0)
                if current_length < length:
                    log.debug("%s length %d < %d modding current column" %
                              (column.name, current_length, length))
                    column.values.append(-1.0)
            length = len(column.values)

    ###
    # Abstract method implementations
    ###

    def get_name(self, set_of_columns=None):
        if set_of_columns is None:
            return self.original_file.name.val[:-4]
        if set_of_columns == 0:
            return self.original_file.name.val[:-4] + ' Cells'
        if set_of_columns == 1:
            return self.original_file.name.val[:-4] + ' Nuclei'
        if set_of_columns == 2:
            return self.original_file.name.val[:-4] + ' Organelles'

    def parse(self):
        log.info("Parsing: %s" % self.original_file.name.val)
        provider = self.original_file_provider
        data = provider.get_original_file_data(self.original_file)
        try:
            events = ('start', 'end')
            well_data = None
            n_roi = 0
            n_measurements = 0
            cells_columns = {'Image': ImageColumn('Image', '', list()),
                             'Cell': LongColumn('Cell', '', list()),
                             'ROI': RoiColumn('ROI', '', list())
                             }
            organelles_columns = {'Image': ImageColumn('Image', '', list()),
                                  'Cell': LongColumn('Cell', '', list()),
                                  }
            nuclei_columns = {'Image': ImageColumn('Image', '', list()),
                              'Cell': LongColumn('Cell', '', list()),
                              'ROI': RoiColumn('ROI', '', list())
                              }
            for event, element in iterparse(data, events=events):
                if event == 'start' and element.tag == 'WellData' \
                   and element.get('cell') != 'Summary':
                    row = int(element.get('row')) - 1
                    col = int(element.get('col')) - 1
                    i = int(element.get('field')) - 1
                    try:
                        well, images = self.get_well_images(row, col)
                        if not images:
                            continue
                        image = images[i]
                    except:
                        log.exception("ERROR: Failed to get well images")
                        continue
                    self.check_sparse_data(list(cells_columns.values()))
                    self.check_sparse_data(list(nuclei_columns.values()))
                    self.check_sparse_data(list(organelles_columns.values()))
                    cell = int(element.get('cell'))
                    cells_columns['Cell'].values.append(cell)
                    nuclei_columns['Cell'].values.append(cell)
                    organelles_columns['Cell'].values.append(cell)
                    well_data = element
                    cells_columns['Image'].values.append(image.id.val)
                    nuclei_columns['Image'].values.append(image.id.val)
                    organelles_columns['Image'].values.append(image.id.val)
                elif well_data is not None and event == 'start' \
                        and element.tag == 'Measure':
                    source = element.get('source')
                    key = element.get('key')
                    value = float(element.get('value'))
                    if source == self.CELLS_SOURCE:
                        columns_list = [cells_columns]
                    elif source == self.NUCLEI_SOURCE:
                        columns_list = [nuclei_columns]
                    elif source == self.ORGANELLES_SOURCE:
                        columns_list = [organelles_columns]
                    else:
                        columns_list = [cells_columns, nuclei_columns,
                                        organelles_columns]
                    for columns in columns_list:
                        if key not in columns:
                            columns[key] = DoubleColumn(key, '', list())
                        columns[key].values.append(value)
                    n_measurements += 1
                elif event == 'end' and element.tag == 'WellData':
                    if well_data is not None:
                        n_roi += 1
                        well_data.clear()
                        well_data = None
                else:
                    element.clear()
            # Final row sparseness check
            self.check_sparse_data(list(cells_columns.values()))
            self.check_sparse_data(list(nuclei_columns.values()))
            self.check_sparse_data(list(organelles_columns.values()))
            log.info("Total ROI: %d" % n_roi)
            log.info("Total measurements: %d" % n_measurements)
            sets_of_columns = [list(cells_columns.values()), list(nuclei_columns.values()),
                               list(organelles_columns.values())]
            return MeasurementParsingResult(sets_of_columns)
        finally:
            data.close()

    def parse_and_populate_roi(self, columns_as_list):
        # First sanity check our provided columns
        names = [column.name for column in columns_as_list]
        log.debug('Parsing columns: %r' % names)
        cells_expected = [name in names for name in self.CELLS_CG_EXPECTED]
        nuclei_expected = [name in names for name in self.NUCLEI_CG_EXPECTED]
        if (False in cells_expected) and (False in nuclei_expected):
            log.warn("Missing CGs for InCell dataset: %r" % names)
            log.warn('Removing resultant empty ROI column.')
            for column in columns_as_list:
                if RoiColumn == column.__class__:
                    columns_as_list.remove(column)
            return
        # Reconstruct a column name to column map
        columns = dict()
        for column in columns_as_list:
            columns[column.name] = column
        image_ids = columns['Image'].values
        rois = list()
        # Save our file annotation to the database so we can use an unloaded
        # annotation for the saveAndReturnIds that will be triggered below.
        self.file_annotation = \
            self.update_service.saveAndReturnObject(self.file_annotation)
        unloaded_file_annotation = \
            FileAnnotationI(self.file_annotation.id.val, False)
        # Parse and append ROI
        batch_no = 1
        batches = dict()
        for i, image_id in enumerate(image_ids):
            unloaded_image = ImageI(image_id, False)
            if False in nuclei_expected:
                # Cell centre of gravity
                roi = RoiI()
                shape = PointI()
                shape.theZ = rint(0)
                shape.theT = rint(0)
                shape.x = rdouble(float(columns['Cell: cgX'].values[i]))
                shape.y = rdouble(float(columns['Cell: cgY'].values[i]))
                roi.addShape(shape)
                roi.image = unloaded_image
                roi.linkAnnotation(unloaded_file_annotation)
                rois.append(roi)
            elif False in cells_expected:
                # Nucleus centre of gravity
                roi = RoiI()
                shape = PointI()
                shape.theZ = rint(0)
                shape.theT = rint(0)
                shape.x = rdouble(float(columns['Nucleus: cgX'].values[i]))
                shape.y = rdouble(float(columns['Nucleus: cgY'].values[i]))
                roi.addShape(shape)
                roi.image = unloaded_image
                roi.linkAnnotation(unloaded_file_annotation)
                rois.append(roi)
            else:
                raise MeasurementError('Not a nucleus or cell ROI')
            if len(rois) == self.ROI_UPDATE_LIMIT:
                self.thread_pool.add_task(
                    self.update_rois, rois, batches, batch_no)
                rois = list()
                batch_no += 1
        self.thread_pool.add_task(self.update_rois, rois, batches, batch_no)
        self.thread_pool.wait_completion()
        batch_keys = list(batches.keys())
        batch_keys.sort()
        for k in batch_keys:
            columns['ROI'].values += batches[k]

    def populate(self, columns):
        self.update_table(columns)

if __name__ == "__main__":
    try:
        options, args = getopt(sys.argv[1:], "s:p:u:m:k:t:id")
    except GetoptError as xxx_todo_changeme:
        (msg, opt) = xxx_todo_changeme.args
        usage(msg)

    try:
        plate_id, = args
        plate_id = int(plate_id)
    except ValueError:
        usage("Plate ID must be a specified and a number!")

    username = None
    hostname = None
    port = 4064  # SSL
    measurement = None
    info = False
    session_key = None
    logging_level = logging.INFO
    thread_count = 1
    for option, argument in options:
        if option == "-u":
            username = argument
        if option == "-s":
            hostname = argument
        if option == "-p":
            port = int(argument)
        if option == "-m":
            measurement = int(argument)
        if option == "-i":
            info = True
        if option == "-k":
            session_key = argument
        if option == "-d":
            logging_level = logging.DEBUG
        if option == "-t":
            thread_count = int(argument)
    if session_key is None and username is None:
        usage("Username must be specified!")
    if session_key is None and hostname is None:
        usage("Host name must be specified!")
    if session_key is None:
        password = getpass()

    logging.basicConfig(level=logging_level)
    c = client(hostname, port)
    c.setAgent("OMERO.populate_roi")
    c.enableKeepAlive(60)
    try:
        if session_key is not None:
            service_factory = c.joinSession(session_key)
        else:
            service_factory = c.createSession(username, password)

        log.debug('Creating pool of %d threads' % thread_count)
        thread_pool = ThreadPool(thread_count)
        factory = PlateAnalysisCtxFactory(service_factory)
        analysis_ctx = factory.get_analysis_ctx(plate_id)
        n_measurements = analysis_ctx.get_measurement_count()
        if measurement is not None and measurement >= n_measurements:
            usage("measurement %d not a valid index!")
        if info:
            for i in range(n_measurements):
                n_result_files = analysis_ctx.get_result_file_count(i)
                print("Measurement %d has %d result files." % \
                    (i, n_result_files))
            sys.exit(0)
        if measurement is not None:
            measurement_ctx = analysis_ctx.get_measurement_ctx(measurement)
            measurement_ctx.parse_and_populate()
        else:
            for i in range(n_measurements):
                measurement_ctx = analysis_ctx.get_measurement_ctx(i)
                measurement_ctx.parse_and_populate()
    finally:
        c.closeSession()
